@@ -408,8 +408,7 @@ static const u_int8_t md5_paddat[MD5_BUFLEN] = {
 
 static void md5_calc(u_int8_t *, md5_ctxt *);
 
-static void md5_init(ctxt)
-	md5_ctxt *ctxt;
+static void md5_init(md5_ctxt *ctxt)
 {
 	ctxt->md5_n = 0;
 	ctxt->md5_i = 0;
@@ -420,10 +419,7 @@ static void md5_init(ctxt)
 	bzero(ctxt->md5_buf, sizeof(ctxt->md5_buf));
 }
 
-static void md5_loop(ctxt, input, len)
-	md5_ctxt *ctxt;
-	const u_int8_t *input;
-	u_int len; /* number of bytes */
+static void md5_loop(md5_ctxt *ctxt, const u_int8_t *input, u_int len)
 {
 	u_int gap, i;
 
@@ -448,8 +444,7 @@ static void md5_loop(ctxt, input, len)
 	}
 }
 
-static void md5_pad(ctxt)
-	md5_ctxt *ctxt;
+static void md5_pad(md5_ctxt *ctxt)
 {
 	u_int gap;
 
@@ -487,9 +482,7 @@ static void md5_pad(ctxt)
 	md5_calc(ctxt->md5_buf, ctxt);
 }
 
-static void md5_result(digest, ctxt)
-	u_int8_t *digest;
-	md5_ctxt *ctxt;
+static void md5_result(u_int8_t *digest, md5_ctxt *ctxt)
 {
 	/* 4 byte words */
 #if BYTE_ORDER == LITTLE_ENDIAN
@@ -507,9 +500,7 @@ static void md5_result(digest, ctxt)
 #endif
 }
 
-static void md5_calc(b64, ctxt)
-	u_int8_t *b64;
-	md5_ctxt *ctxt;
+static void md5_calc(u_int8_t *b64, md5_ctxt *ctxt)
 {
 	u_int32_t A = ctxt->md5_sta;
 	u_int32_t B = ctxt->md5_stb;
