@@ -20,7 +20,13 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_IO_H
+#include <io.h>
+#endif
+#include <string.h>
 #include <memory.h>
 #include <fcntl.h>
 
@@ -74,24 +80,22 @@ void reader_init_filereader(reader_t*r, int handle)
 
 /* ---------------------------- mem reader ------------------------------- */
 
-struct memread_t
+typedef struct _memread
 {
     unsigned char*data;
     int length;
-};
+} memread_t;
+
 static int reader_memread(reader_t*reader, void* data, int len) 
 {
-    struct memread_t*mr = (struct memread_t*)reader->internal;
+    memread_t*mr = (memread_t*)reader->internal;
 
-    if(mr->length - reader->pos > len) {
-	memcpy(data, &mr->data[reader->pos], len);
-	reader->pos += len;
-	return len;
-    } else {
-	memcpy(data, &mr->data[reader->pos], mr->length - reader->pos);
-	reader->pos += mr->length;
-	return mr->length - reader->pos;
+    if(mr->length - reader->pos < len) {
+	len = mr->length - reader->pos;
     }
+    memcpy(data, &mr->data[reader->pos], len);
+    reader->pos += len;
+    return len;
 }
 static void reader_memread_dealloc(reader_t*reader)
 {
@@ -101,8 +105,8 @@ static void reader_memread_dealloc(reader_t*reader)
 }
 void reader_init_memreader(reader_t*r, void*newdata, int newlength)
 {
-    struct memread_t*mr = malloc(sizeof(struct memread_t));
-    mr->data = newdata;
+    memread_t*mr = (memread_t*)malloc(sizeof(memread_t));
+    mr->data = (unsigned char*)newdata;
     mr->length = newlength;
     r->read = reader_memread;
     r->dealloc = reader_memread_dealloc;
@@ -115,15 +119,15 @@ void reader_init_memreader(reader_t*r, void*newdata, int newlength)
 
 /* ---------------------------- mem writer ------------------------------- */
 
-struct memwrite_t
+typedef struct _memwrite
 {
     unsigned char*data;
     int length;
-};
+} memwrite_t;
 
 static int writer_memwrite_write(writer_t*w, void* data, int len) 
 {
-    struct memwrite_t*mw = (struct memwrite_t*)w->internal;
+    memwrite_t*mw = (memwrite_t*)w->internal;
     if(mw->length - w->pos > len) {
 	memcpy(&mw->data[w->pos], data, len);
 	w->pos += len;
@@ -140,14 +144,18 @@ static void writer_memwrite_finish(writer_t*w)
 	free(w->internal);
     w->internal = 0;
 }
+static void dummy_flush(writer_t*w)
+{
+}
 void writer_init_memwriter(writer_t*w, void*data, int len)
 {
-    struct memwrite_t *mr;
-    mr = malloc(sizeof(struct memwrite_t));
-    mr->data = data;
+    memwrite_t *mr;
+    mr = (memwrite_t*)malloc(sizeof(memwrite_t));
+    mr->data = (unsigned char *)data;
     mr->length = len;
     memset(w, 0, sizeof(writer_t));
     w->write = writer_memwrite_write;
+    w->flush = dummy_flush;
     w->finish = writer_memwrite_finish;
     w->internal = (void*)mr;
     w->type = WRITER_TYPE_MEM;
@@ -158,32 +166,31 @@ void writer_init_memwriter(writer_t*w, void*data, int len)
 
 /* ------------------------- growing mem writer ------------------------------- */
 
-struct growmemwrite_t
+typedef struct _growmemwrite
 {
     unsigned char*data;
     int length;
     U32 grow;
-};
+} growmemwrite_t;
 static int writer_growmemwrite_write(writer_t*w, void* data, int len) 
 {
-    struct growmemwrite_t*mw = (struct growmemwrite_t*)w->internal;
+    growmemwrite_t*mw = (growmemwrite_t*)w->internal;
     if(!mw->data) {
 	fprintf(stderr, "Illegal write operation: data already given away");
 	exit(1);
     }
     if(mw->length - w->pos < len) {
-	unsigned char*newmem;
 	int newlength = mw->length;
 	while(newlength - w->pos < len) {
 	    newlength += mw->grow;
 	}
 #ifdef NO_REALLOC
-	newmem = malloc(newlength);
+	unsigned char*newmem = (unsigned char*)malloc(newlength);
 	memcpy(newmem, mw->data, mw->length);
 	free(mw->data);
 	mw->data = newmem;
 #else
-	mw->data = realloc(mw->data, newlength);
+	mw->data = (unsigned char*)realloc(mw->data, newlength);
 #endif
 	mw->length = newlength;
     }
@@ -193,17 +200,25 @@ static int writer_growmemwrite_write(writer_t*w, void* data, int len)
 }
 static void writer_growmemwrite_finish(writer_t*w)
 {
-    struct growmemwrite_t*mw = (struct growmemwrite_t*)w->internal;
-    if(mw->data)
-	free(mw->data);
-    mw->data = 0;
+    growmemwrite_t*mw = (growmemwrite_t*)w->internal;
+    if(mw->data) {
+	free(mw->data);mw->data = 0;
+    }
     mw->length = 0;
     free(w->internal);mw=0;
     memset(w, 0, sizeof(writer_t));
 }
+void* writer_growmemwrite_memptr(writer_t*w, int*len)
+{
+    growmemwrite_t*mw = (growmemwrite_t*)w->internal;
+    if(len) {
+	*len = w->pos;
+    }
+    return mw->data;
+}
 void* writer_growmemwrite_getmem(writer_t*w)
 {
-    struct growmemwrite_t*mw = (struct growmemwrite_t*)w->internal;
+    growmemwrite_t*mw = (growmemwrite_t*)w->internal;
     void*ret = mw->data;
     /* remove own reference so that neither write() nor finish() can free it.
        It's property of the caller now.
@@ -211,17 +226,24 @@ void* writer_growmemwrite_getmem(writer_t*w)
     mw->data = 0;
     return ret;
 }
+void writer_growmemwrite_reset(writer_t*w)
+{
+    growmemwrite_t*mw = (growmemwrite_t*)w->internal;
+    w->pos = 0;
+    w->bitpos = 0;
+    w->mybyte = 0;
+}
 void writer_init_growingmemwriter(writer_t*w, U32 grow)
 {
-    struct growmemwrite_t *mr;
-    mr = malloc(sizeof(struct growmemwrite_t));
-    mr->length = 4096;
-    mr->data = malloc(mr->length);
-    mr->grow = grow;
+    growmemwrite_t *mw = (growmemwrite_t *)malloc(sizeof(growmemwrite_t));
+    mw->length = 4096;
+    mw->data = (unsigned char *)malloc(mw->length);
+    mw->grow = grow;
     memset(w, 0, sizeof(writer_t));
     w->write = writer_growmemwrite_write;
+    w->flush = dummy_flush;
     w->finish = writer_growmemwrite_finish;
-    w->internal = (void*)mr;
+    w->internal = (void*)mw;
     w->type = WRITER_TYPE_GROWING_MEM;
     w->bitpos = 0;
     w->mybyte = 0;
@@ -230,20 +252,21 @@ void writer_init_growingmemwriter(writer_t*w, U32 grow)
 
 /* ---------------------------- file writer ------------------------------- */
 
-struct filewrite_t
+typedef struct _filewrite
 {
     int handle;
     char free_handle;
-};
+} filewrite_t;
 
 static int writer_filewrite_write(writer_t*w, void* data, int len) 
 {
-    struct filewrite_t * fw= (struct filewrite_t*)w->internal;
+    filewrite_t * fw= (filewrite_t*)w->internal;
+    w->pos += len;
     return write(fw->handle, data, len);
 }
 static void writer_filewrite_finish(writer_t*w)
 {
-    struct filewrite_t *mr = (struct filewrite_t*)w->internal;
+    filewrite_t *mr = (filewrite_t*)w->internal;
     if(mr->free_handle)
 	close(mr->handle);
     free(w->internal);
@@ -251,7 +274,7 @@ static void writer_filewrite_finish(writer_t*w)
 }
 void writer_init_filewriter(writer_t*w, int handle)
 {
-    struct filewrite_t *mr = malloc(sizeof(struct filewrite_t));
+    filewrite_t *mr = (filewrite_t *)malloc(sizeof(filewrite_t));
     mr->handle = handle;
     mr->free_handle = 0;
     memset(w, 0, sizeof(writer_t));
@@ -271,7 +294,7 @@ void writer_init_filewriter2(writer_t*w, char*filename)
 #endif
 	    O_WRONLY|O_CREAT|O_TRUNC, 0644);
     writer_init_filewriter(w, fi);
-    ((struct filewrite_t*)w->internal)->free_handle = 1;
+    ((filewrite_t*)w->internal)->free_handle = 1;
 }
 
 /* ---------------------------- null writer ------------------------------- */
@@ -289,6 +312,7 @@ void writer_init_nullwriter(writer_t*w)
 {
     memset(w, 0, sizeof(writer_t));
     w->write = writer_nullwrite_write;
+    w->flush = dummy_flush;
     w->finish = writer_nullwrite_finish;
     w->internal = 0;
     w->type = WRITER_TYPE_NULL;
@@ -298,14 +322,14 @@ void writer_init_nullwriter(writer_t*w)
 }
 /* ---------------------------- zlibinflate reader -------------------------- */
 
-struct zlibinflate_t
+typedef struct _zlibinflate
 {
 #ifdef HAVE_ZLIB
     z_stream zs;
     reader_t*input;
     unsigned char readbuffer[ZLIB_BUFFER_SIZE];
 #endif
-};
+} zlibinflate_t;
 
 #ifdef HAVE_ZLIB
 static void zlib_error(int ret, char* msg, z_stream*zs)
@@ -322,7 +346,7 @@ static void zlib_error(int ret, char* msg, z_stream*zs)
 static int reader_zlibinflate(reader_t*reader, void* data, int len) 
 {
 #ifdef HAVE_ZLIB
-    struct zlibinflate_t*z = (struct zlibinflate_t*)reader->internal;
+    zlibinflate_t*z = (zlibinflate_t*)reader->internal;
     int ret;
     if(!z) {
 	return 0;
@@ -330,7 +354,7 @@ static int reader_zlibinflate(reader_t*reader, void* data, int len)
     if(!len)
 	return 0;
     
-    z->zs.next_out = data;
+    z->zs.next_out = (Bytef *)data;
     z->zs.avail_out = len;
 
     while(1) {
@@ -369,7 +393,7 @@ static int reader_zlibinflate(reader_t*reader, void* data, int len)
 static void reader_zlibinflate_dealloc(reader_t*reader)
 {
 #ifdef HAVE_ZLIB
-    struct zlibinflate_t*z = (struct zlibinflate_t*)reader->internal;
+    zlibinflate_t*z = (zlibinflate_t*)reader->internal;
     /* test whether read() already did basic deallocation */
     if(reader->internal) {
 	inflateEnd(&z->zs);
@@ -381,11 +405,10 @@ static void reader_zlibinflate_dealloc(reader_t*reader)
 void reader_init_zlibinflate(reader_t*r, reader_t*input)
 {
 #ifdef HAVE_ZLIB
-    struct zlibinflate_t*z;
+    zlibinflate_t*z = (zlibinflate_t*)malloc(sizeof(zlibinflate_t));
+    memset(z, 0, sizeof(zlibinflate_t));
     int ret;
     memset(r, 0, sizeof(reader_t));
-    z = (struct zlibinflate_t*)malloc(sizeof(struct zlibinflate_t));
-    memset(z, 0, sizeof(struct zlibinflate_t));
     r->internal = z;
     r->read = reader_zlibinflate;
     r->dealloc = reader_zlibinflate_dealloc;
@@ -407,19 +430,19 @@ void reader_init_zlibinflate(reader_t*r, reader_t*input)
 
 /* ---------------------------- zlibdeflate writer -------------------------- */
 
-struct zlibdeflate_t
+typedef struct _zlibdeflate
 {
 #ifdef HAVE_ZLIB
     z_stream zs;
     writer_t*output;
     unsigned char writebuffer[ZLIB_BUFFER_SIZE];
 #endif
-};
+} zlibdeflate_t;
 
 static int writer_zlibdeflate_write(writer_t*writer, void* data, int len) 
 {
 #ifdef HAVE_ZLIB
-    struct zlibdeflate_t*z = (struct zlibdeflate_t*)writer->internal;
+    zlibdeflate_t*z = (zlibdeflate_t*)writer->internal;
     int ret;
     if(writer->type != WRITER_TYPE_ZLIB) {
 	fprintf(stderr, "Wrong writer ID (writer not initialized?)\n");
@@ -432,7 +455,7 @@ static int writer_zlibdeflate_write(writer_t*writer, void* data, int len)
     if(!len)
 	return 0;
     
-    z->zs.next_in = data;
+    z->zs.next_in = (Bytef *)data;
     z->zs.avail_in = len;
 
     while(1) {
@@ -441,6 +464,7 @@ static int writer_zlibdeflate_write(writer_t*writer, void* data, int len)
 	if (ret != Z_OK) zlib_error(ret, "bitio:deflate_deflate", &z->zs);
 
 	if(z->zs.next_out != z->writebuffer) {
+	    writer->pos += z->zs.next_out - (Bytef*)z->writebuffer;
 	    z->output->write(z->output, z->writebuffer, z->zs.next_out - (Bytef*)z->writebuffer);
 	    z->zs.next_out = z->writebuffer;
 	    z->zs.avail_out = ZLIB_BUFFER_SIZE;
@@ -450,17 +474,52 @@ static int writer_zlibdeflate_write(writer_t*writer, void* data, int len)
 	    break;
 	}
     }
-    writer->pos += len;
     return len;
 #else
     fprintf(stderr, "Error: swftools was compiled without zlib support");
     exit(1);
 #endif
 }
+
+void writer_zlibdeflate_flush(writer_t*writer)
+{
+#ifdef HAVE_ZLIB
+    zlibdeflate_t*z = (zlibdeflate_t*)writer->internal;
+    int ret;
+    if(writer->type != WRITER_TYPE_ZLIB) {
+	fprintf(stderr, "Wrong writer ID (writer not initialized?)\n");
+	return;
+    }
+    if(!z) {
+	fprintf(stderr, "zlib not initialized!\n");
+	return;
+    }
+
+    z->zs.next_in = 0;
+    z->zs.avail_in = 0;
+    while(1) {
+	ret = deflate(&z->zs, Z_SYNC_FLUSH);
+	if (ret != Z_OK) zlib_error(ret, "bitio:deflate_flush", &z->zs);
+	if(z->zs.next_out != z->writebuffer) {
+	    writer->pos += z->zs.next_out - (Bytef*)z->writebuffer;
+	    z->output->write(z->output, z->writebuffer, z->zs.next_out - (Bytef*)z->writebuffer);
+	    z->zs.next_out = z->writebuffer;
+	    z->zs.avail_out = ZLIB_BUFFER_SIZE;
+	} 
+	/* TODO: how will zlib let us know it needs more buffer space? */
+	break;
+    }
+    return;
+#else
+    fprintf(stderr, "Error: swftools was compiled without zlib support");
+    exit(1);
+#endif
+}
+
 static void writer_zlibdeflate_finish(writer_t*writer)
 {
 #ifdef HAVE_ZLIB
-    struct zlibdeflate_t*z = (struct zlibdeflate_t*)writer->internal;
+    zlibdeflate_t*z = (zlibdeflate_t*)writer->internal;
     writer_t*output;
     int ret;
     if(writer->type != WRITER_TYPE_ZLIB) {
@@ -476,6 +535,7 @@ static void writer_zlibdeflate_finish(writer_t*writer)
 	    ret != Z_STREAM_END) zlib_error(ret, "bitio:deflate_finish", &z->zs);
 
 	if(z->zs.next_out != z->writebuffer) {
+	    writer->pos += z->zs.next_out - (Bytef*)z->writebuffer;
 	    z->output->write(z->output, z->writebuffer, z->zs.next_out - (Bytef*)z->writebuffer);
 	    z->zs.next_out = z->writebuffer;
 	    z->zs.avail_out = ZLIB_BUFFER_SIZE;
@@ -499,13 +559,14 @@ static void writer_zlibdeflate_finish(writer_t*writer)
 void writer_init_zlibdeflate(writer_t*w, writer_t*output)
 {
 #ifdef HAVE_ZLIB
-    struct zlibdeflate_t*z;
+    zlibdeflate_t*z;
     int ret;
     memset(w, 0, sizeof(writer_t));
-    z = (struct zlibdeflate_t*)malloc(sizeof(struct zlibdeflate_t));
-    memset(z, 0, sizeof(struct zlibdeflate_t));
+    z = (zlibdeflate_t*)malloc(sizeof(zlibdeflate_t));
+    memset(z, 0, sizeof(zlibdeflate_t));
     w->internal = z;
     w->write = writer_zlibdeflate_write;
+    w->flush = writer_zlibdeflate_flush;
     w->finish = writer_zlibdeflate_finish;
     w->type = WRITER_TYPE_ZLIB;
     w->pos = 0;
@@ -585,29 +646,43 @@ void reader_resetbits(reader_t*r)
 
 U8 reader_readU8(reader_t*r)
 {
-    U8 b;
-    r->read(r, &b, 1);
+    U8 b = 0;
+    if(r->read(r, &b, 1)<1) {
+	fprintf(stderr, "bitio.c:reader_readU8: Read over end of memory region\n");
+    }
     return b;
 }
 U16 reader_readU16(reader_t*r)
 {
-    U8 b1,b2;
-    r->read(r, &b1, 1);
-    r->read(r, &b2, 1);
+    U8 b1=0,b2=0;
+    if(r->read(r, &b1, 1)<1) {
+	fprintf(stderr, "bitio.c:reader_readU16: Read over end of memory region\n");
+    }
+    if(r->read(r, &b2, 1)<1) {
+	fprintf(stderr, "bitio.c:reader_readU16: Read over end of memory region\n");
+    }
     return b1|b2<<8;
 }
 U32 reader_readU32(reader_t*r)
 {
-    U8 b1,b2,b3,b4;
-    r->read(r, &b1, 1);
-    r->read(r, &b2, 1);
-    r->read(r, &b3, 1);
-    r->read(r, &b4, 1);
+    U8 b1=0,b2=0,b3=0,b4=0;
+    if(r->read(r, &b1, 1)<1)
+	fprintf(stderr, "bitio.c:reader_readU32: Read over end of memory region\n");
+    if(r->read(r, &b2, 1)<1)
+	fprintf(stderr, "bitio.c:reader_readU32: Read over end of memory region\n");
+    if(r->read(r, &b3, 1)<1)
+	fprintf(stderr, "bitio.c:reader_readU32: Read over end of memory region\n");
+    if(r->read(r, &b4, 1)<1)
+	fprintf(stderr, "bitio.c:reader_readU32: Read over end of memory region\n");
     return b1|b2<<8|b3<<16|b4<<24;
 }
 float reader_readFloat(reader_t*r)
 {
-    U8 b1,b2,b3,b4;
+    float f;
+    r->read(r, &f, 4);
+    return f;
+
+    U8 b1=0,b2=0,b3=0,b4=0;
     r->read(r, &b1, 1);
     r->read(r, &b2, 1);
     r->read(r, &b3, 1);
@@ -636,8 +711,8 @@ char*reader_readString(reader_t*r)
 	if(!b)
 	    break;
     }
-    char*string = writer_growmemwrite_getmem(&g);
-    writer_growmemwrite_finish(&g);
+    char*string = (char*)writer_growmemwrite_getmem(&g);
+    g.finish(&g);
     return string;
 }
 
@@ -672,7 +747,10 @@ void writer_writeU32(writer_t*w, unsigned long v)
 }
 void writer_writeFloat(writer_t*w, float f)
 {
-    unsigned long v = *(unsigned long*)&f;
+    w->write(w, &f, 4);
+    return;
+
+    unsigned v = *(unsigned*)&f;
     unsigned char b1 = v;
     unsigned char b2 = v>>8;
     unsigned char b3 = v>>16;
@@ -705,5 +783,3 @@ void writer_writeDouble(writer_t*w, double f)
     w->write(w, &b7, 1);
     w->write(w, &b8, 1);
 }
-
-
