@@ -21,6 +21,30 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
+#include <stdlib.h>
+#include <stdio.h>
+#include <memory.h>
+#include "../../config.h"
+#ifdef HAVE_ZLIB
+#include <zconf.h>
+#include <zlib.h>
+#endif
+#include <fcntl.h>
+#include <ctype.h>
+
+#ifdef HAVE_JPEGLIB
+#define HAVE_BOOLEAN
+#ifdef __cplusplus
+extern "C" {
+#endif
+#include <jpeglib.h>
+#ifdef __cplusplus
+}
+#endif
+#endif // HAVE_JPEGLIB
+
+#include "../rfxswf.h"
+
 #define OUTBUFFER_SIZE 0x8000
 
 int swf_ImageHasAlpha(RGBA*img, int width, int height)
@@ -112,7 +136,7 @@ int swf_ImageGetNumberOfPaletteEntries(RGBA*img, int width, int height, RGBA*pal
     int palette_overflow = 0;
     U32 lastcol32 = 0;
 
-    pal = malloc(65536*sizeof(U32));
+    pal = (U32*)malloc(65536*sizeof(U32));
 
     memset(size, 0, sizeof(size));
 
@@ -270,18 +294,19 @@ int swf_SetJPEGBitsFinish(JPEGBITS * jpegbits)
     if (!jpeg)
 	return -1;
     jpeg_finish_compress(&jpeg->cinfo);
+    jpeg_destroy_compress(&jpeg->cinfo);
     rfx_free(jpeg);
     return 0;
 }
 
-void swf_SetJPEGBits2(TAG * tag, U16 width, U16 height, RGBA * bitmap,
-		      int quality)
+#if defined(HAVE_JPEGLIB)
+void swf_SetJPEGBits2(TAG * tag, U16 width, U16 height, RGBA * bitmap, int quality)
 {
     JPEGBITS *jpeg;
     int y;
     jpeg = swf_SetJPEGBitsStart(tag, width, height, quality);
+	U8 *scanline = (U8*)rfx_alloc(3 * width);
     for (y = 0; y < height; y++) {
-	U8 scanline[3 * width];
 	int x, p = 0;
 	for (x = 0; x < width; x++) {
 	    scanline[p++] = bitmap[width * y + x].r;
@@ -290,10 +315,18 @@ void swf_SetJPEGBits2(TAG * tag, U16 width, U16 height, RGBA * bitmap,
 	}
 	swf_SetJPEGBitsLine(jpeg, scanline);
     }
+    rfx_free(scanline);
     swf_SetJPEGBitsFinish(jpeg);
 }
+#else
+void swf_SetJPEGBits2(TAG * tag, U16 width, U16 height, RGBA * bitmap, int quality)
+{
+    fprintf(stderr, "Error: swftools compiled without jpeglib\n");
+    return -1;
+}
+#endif
 
-void swf_GetJPEGSize(char *fname, int *width, int *height)
+void swf_GetJPEGSize(const char *fname, int *width, int *height)
 {
     struct jpeg_decompress_struct cinfo;
     struct jpeg_error_mgr jerr;
@@ -314,7 +347,7 @@ void swf_GetJPEGSize(char *fname, int *width, int *height)
     fclose(fi);
 }
 
-int swf_SetJPEGBits(TAG * t, char *fname, int quality)
+int swf_SetJPEGBits(TAG * t, const char *fname, int quality)
 {
     struct jpeg_decompress_struct cinfo;
     struct jpeg_error_mgr jerr;
@@ -393,6 +426,7 @@ int swf_SetJPEGBits(TAG * t, char *fname, int quality)
 	}
     }
 
+    rfx_free(scanline);
     swf_SetJPEGBitsFinish(out);
     jpeg_finish_decompress(&cinfo);
     fclose(f);
@@ -466,7 +500,7 @@ void swf_SaveJPEG(char*filename, RGBA*pixels, int width, int height, int quality
 	perror(buf);
 	return;
     }
-    data2 = rfx_calloc(width*3);
+    data2 = (unsigned char *)rfx_calloc(width*3);
 
     memset(&cinfo, 0, sizeof(cinfo));
     memset(&jerr, 0, sizeof(jerr));
@@ -600,7 +634,7 @@ RGBA *swf_JPEG2TagToImage(TAG * tag, int *width, int *height)
     jpeg_read_header(&cinfo, TRUE);
     *width = cinfo.image_width;
     *height = cinfo.image_height;
-    dest =
+    dest = (RGBA*)
 	rfx_alloc(sizeof(RGBA) * cinfo.image_width * cinfo.image_height);
 
     jpeg_start_decompress(&cinfo);
@@ -626,7 +660,7 @@ RGBA *swf_JPEG2TagToImage(TAG * tag, int *width, int *height)
 
 #ifdef HAVE_ZLIB
     if(offset) {
-	U32 datalen = cinfo.output_width*cinfo.output_height;
+	uLongf datalen = cinfo.output_width*cinfo.output_height;
 	U8* alphadata = (U8*)rfx_alloc(datalen);
 	int error;
 	tag->len = oldtaglen;
@@ -658,7 +692,7 @@ RGBA *swf_JPEG2TagToImage(TAG * tag, int *width, int *height)
 
 int RFXSWF_deflate_wraper(TAG * t, z_stream * zs, boolean finish)
 {
-    U8 *data = rfx_alloc(OUTBUFFER_SIZE);
+    U8 *data = (U8*)rfx_alloc(OUTBUFFER_SIZE);
     zs->next_out = data;
     zs->avail_out = OUTBUFFER_SIZE;
     while (1) {
@@ -714,7 +748,7 @@ int swf_SetLosslessBits(TAG * t, U16 width, U16 height, void *bitmap, U8 bitmap_
 
     switch (bitmap_flags) {
     case BMF_8BIT:
-	return swf_SetLosslessBitsIndexed(t, width, height, bitmap, NULL, 256);
+	return swf_SetLosslessBitsIndexed(t, width, height, (U8*)bitmap, NULL, 256);
     case BMF_16BIT:
 	bps = BYTES_PER_SCANLINE(sizeof(U16) * width);
 	break;
@@ -739,7 +773,7 @@ int swf_SetLosslessBits(TAG * t, U16 width, U16 height, void *bitmap, U8 bitmap_
 
 	if (deflateInit(&zs, Z_DEFAULT_COMPRESSION) == Z_OK) {
 	    zs.avail_in = bps * height;
-	    zs.next_in = bitmap;
+	    zs.next_in = (Bytef *)bitmap;
 
 	    if (RFXSWF_deflate_wraper(t, &zs, TRUE) < 0)
 		res = -3;
@@ -760,7 +794,7 @@ int swf_SetLosslessBitsIndexed(TAG * t, U16 width, U16 height, U8 * bitmap, RGBA
     if (!pal)			// create default palette for grayscale images
     {
 	int i;
-	pal = rfx_alloc(256 * sizeof(RGBA));
+	pal = (RGBA*)rfx_alloc(256 * sizeof(RGBA));
 	for (i = 0; i < 256; i++) {
 	    pal[i].r = pal[i].g = pal[i].b = i;
 	    pal[i].a = 0xff;
@@ -788,7 +822,7 @@ int swf_SetLosslessBitsIndexed(TAG * t, U16 width, U16 height, U8 * bitmap, RGBA
 
 	if (deflateInit(&zs, Z_DEFAULT_COMPRESSION) == Z_OK) {
 	    U8 *zpal;		// compress palette
-	    if ((zpal = rfx_alloc(ncolors * 4))) {
+	    if ((zpal = (U8*)rfx_alloc(ncolors * 4))) {
 		U8 *pp = zpal;
 		int i;
 
@@ -867,6 +901,7 @@ void swf_PreMultiplyAlpha(RGBA*data, int width, int height)
     }
 }
 
+/* expects mem to be non-premultiplied */
 void swf_SetLosslessImage(TAG*tag, RGBA*data, int width, int height)
 {
     int hasalpha = swf_ImageHasAlpha(data, width, height);
@@ -915,7 +950,7 @@ void swf_SetLosslessImage(TAG*tag, RGBA*data, int width, int height)
 RGBA *swf_DefineLosslessBitsTagToImage(TAG * tag, int *dwidth, int *dheight)
 {
     int id, format, height, width, pos;
-    U32 datalen, datalen2;
+    uLongf datalen, datalen2;
     int error;
     int bpp = 1;
     int cols = 0;
@@ -953,7 +988,7 @@ RGBA *swf_DefineLosslessBitsTagToImage(TAG * tag, int *dwidth, int *dheight)
     *dwidth = width = swf_GetU16(tag);
     *dheight = height = swf_GetU16(tag);
 
-    dest = rfx_alloc(sizeof(RGBA) * width * height);
+    dest = (RGBA*)rfx_alloc(sizeof(RGBA) * width * height);
 
     if (format == 3)
 	cols = swf_GetU8(tag) + 1;
@@ -966,7 +1001,7 @@ RGBA *swf_DefineLosslessBitsTagToImage(TAG * tag, int *dwidth, int *dheight)
 	if (data)
 	    rfx_free(data);
 	datalen += 4096;
-	data = rfx_alloc(datalen);
+	data = (U8*)rfx_alloc(datalen);
 	error =
 	    uncompress(data, &datalen, &tag->data[tag->pos],
 		       tag->len - tag->pos);
@@ -1006,20 +1041,13 @@ RGBA *swf_DefineLosslessBitsTagToImage(TAG * tag, int *dwidth, int *dheight)
 		}
 	    } else {
 		for (x = 0; x < width; x++) {
-		    /* TODO: is un-premultiplying alpha the right thing to do?
-		       dest[pos2].r = data[pos + 1];
-		       dest[pos2].g = data[pos + 2];
-		       dest[pos2].b = data[pos + 3];*/
+		    /* remove premultiplication */
 		    int alpha = data[pos+0];
-		    if(alpha) {
-			dest[pos2].r = ((int)data[pos + 1]*255)/alpha;
-			dest[pos2].g = ((int)data[pos + 2]*255)/alpha;
-			dest[pos2].b = ((int)data[pos + 3]*255)/alpha;
-		    } else {
-			dest[pos2].r = data[pos + 1];
-			dest[pos2].g = data[pos + 2];
-			dest[pos2].b = data[pos + 3];
-		    }
+		    if(alpha)
+			alpha = 0xff0000/alpha;
+		    dest[pos2].r = (data[pos + 1]*alpha)>>16;
+		    dest[pos2].g = (data[pos + 2]*alpha)>>16;
+		    dest[pos2].b = (data[pos + 3]*alpha)>>16;
 		    dest[pos2].a = data[pos + 0];	//alpha
 		    pos2++;
 		    pos += 4;
@@ -1042,6 +1070,8 @@ RGBA *swf_DefineLosslessBitsTagToImage(TAG * tag, int *dwidth, int *dheight)
 #endif				// HAVE_ZLIB
 
 #if defined(HAVE_ZLIB) && defined(HAVE_JPEGLIB)
+
+/* expects bitmap to be non-premultiplied */
 int swf_SetJPEGBits3(TAG * tag, U16 width, U16 height, RGBA * bitmap, int quality)
 {
     JPEGBITS *jpeg;
@@ -1054,20 +1084,29 @@ int swf_SetJPEGBits3(TAG * tag, U16 width, U16 height, RGBA * bitmap, int qualit
     pos = tag->len;
     swf_SetU32(tag, 0);		//placeholder
     jpeg = swf_SetJPEGBitsStart(tag, width, height, quality);
+	U8 *scanline = (U8*)rfx_alloc(3 * width);
     for (y = 0; y < height; y++) {
-	U8 scanline[3 * width];
 	int x, p = 0;
 	for (x = 0; x < width; x++) {
+	    //int ia = bitmap[width*y+x].a;
+	    //if(ia) {
+	    //    /* remove premultiplication */
+	    //    ia = 0xff0000/ia;
+	    //}
+	    //scanline[p++] = (bitmap[width * y + x].r*ia)>>16;
+	    //scanline[p++] = (bitmap[width * y + x].g*ia)>>16;
+	    //scanline[p++] = (bitmap[width * y + x].b*ia)>>16;
 	    scanline[p++] = bitmap[width * y + x].r;
 	    scanline[p++] = bitmap[width * y + x].g;
 	    scanline[p++] = bitmap[width * y + x].b;
 	}
 	swf_SetJPEGBitsLine(jpeg, scanline);
     }
+    rfx_free(scanline);
     swf_SetJPEGBitsFinish(jpeg);
     PUT32(&tag->data[pos], tag->len - pos - 4);
 
-    data = rfx_alloc(OUTBUFFER_SIZE);
+    data = (U8*)rfx_alloc(OUTBUFFER_SIZE);
     memset(&zs, 0x00, sizeof(z_stream));
 
     if (deflateInit(&zs, Z_DEFAULT_COMPRESSION) != Z_OK) {
@@ -1078,8 +1117,8 @@ int swf_SetJPEGBits3(TAG * tag, U16 width, U16 height, RGBA * bitmap, int qualit
     zs.next_out = data;
     zs.avail_out = OUTBUFFER_SIZE;
 
+	scanline = (U8*)rfx_alloc(width);
     for (y = 0; y < height; y++) {
-	U8 scanline[width];
 	int x, p = 0;
 	for (x = 0; x < width; x++) {
 	    scanline[p++] = bitmap[width * y + x].a;
@@ -1103,6 +1142,8 @@ int swf_SetJPEGBits3(TAG * tag, U16 width, U16 height, RGBA * bitmap, int qualit
 	}
     }
 
+    rfx_free(scanline);
+
     while (1) {
 	int ret = deflate(&zs, Z_FINISH);
 	if (ret != Z_OK && ret != Z_STREAM_END) {
@@ -1124,6 +1165,16 @@ int swf_SetJPEGBits3(TAG * tag, U16 width, U16 height, RGBA * bitmap, int qualit
     return 0;
 }
 
+#else
+int swf_SetJPEGBits3(TAG * tag, U16 width, U16 height, RGBA * bitmap, int quality)
+{
+    fprintf(stderr, "Error: swftools compiled without jpeglib\n");
+    return -1;
+}
+#endif
+
+
+/* expects mem to be non-premultiplied */
 TAG* swf_AddImage(TAG*tag, int bitid, RGBA*mem, int width, int height, int quality)
 {
     TAG *tag1 = 0, *tag2 = 0;
@@ -1134,6 +1185,7 @@ TAG* swf_AddImage(TAG*tag, int bitid, RGBA*mem, int width, int height, int quali
     swf_SetU16(tag1, bitid);
     swf_SetLosslessImage(tag1, mem, width, height);
 
+#if defined(HAVE_JPEGLIB)
     /* try jpeg image */
     if(has_alpha) {
 	tag2 = swf_InsertTag(0, ST_DEFINEBITSJPEG3);
@@ -1144,25 +1196,24 @@ TAG* swf_AddImage(TAG*tag, int bitid, RGBA*mem, int width, int height, int quali
 	swf_SetU16(tag2, bitid);
 	swf_SetJPEGBits2(tag2, width, height, mem, quality);
     }
+#endif
 
-    if(tag1 && tag1->len < tag2->len) {
+    if(quality>100 || !tag2 || (tag1 && tag1->len < tag2->len)) {
 	/* use the zlib version- it's smaller */
 	tag1->prev = tag;
 	if(tag) tag->next = tag1;
 	tag = tag1;
-	swf_DeleteTag(tag2);
+	swf_DeleteTag(0, tag2);
     } else {
 	/* use the jpeg version- it's smaller */
 	tag2->prev = tag;
 	if(tag) tag->next = tag2;
 	tag = tag2;
-	swf_DeleteTag(tag1);
+	swf_DeleteTag(0, tag1);
     }
     return tag;
 }
 
-#endif
-
 RGBA *swf_ExtractImage(TAG * tag, int *dwidth, int *dheight)
 {
     RGBA *img;
@@ -1214,7 +1265,7 @@ void swf_RemoveJPEGTables(SWF * swf)
 	if (tag->id == ST_DEFINEBITSJPEG) {
 	    int len = tag->len;
 	    void *data = rfx_alloc(len);
-	    swf_GetBlock(tag, data, tag->len);
+	    swf_GetBlock(tag, (U8*)data, tag->len);
 	    swf_ResetTag(tag, ST_DEFINEBITSJPEG2);
 	    swf_SetBlock(tag, &((U8*)data)[0], 2); //id
 	    swf_SetBlock(tag, tables_tag->data, tables_tag->len);
@@ -1225,7 +1276,7 @@ void swf_RemoveJPEGTables(SWF * swf)
     }
     if (swf->firstTag == tables_tag)
 	swf->firstTag = tables_tag->next;
-    swf_DeleteTag(tables_tag);
+    swf_DeleteTag(swf, tables_tag);
 }
 
 typedef struct scale_lookup {
@@ -1241,7 +1292,7 @@ static int bicubic = 0;
 
 static scale_lookup_t**make_scale_lookup(int width, int newwidth)
 {
-    scale_lookup_t*lookupx = malloc((width>newwidth?width:newwidth)*2*sizeof(scale_lookup_t));
+    scale_lookup_t*lookupx = (scale_lookup_t*)malloc((width>newwidth?width:newwidth)*2*sizeof(scale_lookup_t));
     scale_lookup_t**lblockx = (scale_lookup_t**)malloc((newwidth+1)*sizeof(scale_lookup_t**));
     double fx = ((double)width)/((double)newwidth);
     double px = 0;
@@ -1331,6 +1382,97 @@ static void decodeMonochromeImage(RGBA*data, int width, int height, RGBA*colors)
     }
 }
 
+static void blurImage(RGBA*src, int width, int height, int r)
+{
+    int e = 2; // r times e is the sampling interval
+    double*gauss = (double*)malloc(r*e*sizeof(double));
+    double sum=0;
+    int x;
+    for(x=0;x<r*e;x++) {
+        double t = (x - r*e/2.0)/r;
+        gauss[x] = exp(-0.5*t*t);
+        sum += gauss[x];
+    }
+    int*weights = (int*)malloc(r*e*sizeof(int));
+    for(x=0;x<r*e;x++) {
+        weights[x] = (int)(gauss[x]*65536.0001/sum);
+    }
+    int range = r*e/2;
+
+    RGBA*tmp = malloc(sizeof(RGBA)*width*height);
+
+    int y;
+    for(y=0;y<height;y++) {
+        RGBA*s = &src[y*width];
+        RGBA*d = &tmp[y*width];
+        for(x=0;x<range;x++) {
+            d[x] = s[x];
+        }
+        for(x=range;x<width-range;x++) {
+            int r=0;
+            int g=0;
+            int b=0;
+            int a=0;
+            int*f = weights;
+            int xx;
+            for(xx=x-range;xx<x+range;xx++) {
+                r += s[xx].r * f[0];
+                g += s[xx].g * f[0];
+                b += s[xx].b * f[0];
+                a += s[xx].a * f[0];
+                f++;
+            }
+            d[x].r = r >> 16;
+            d[x].g = g >> 16;
+            d[x].b = b >> 16;
+            d[x].a = a >> 16;
+        }
+        for(x=width-range;x<width;x++) {
+            d[x] = s[x];
+        }
+    }
+
+    for(x=0;x<width;x++) {
+        RGBA*s = &tmp[x];
+        RGBA*d = &src[x];
+        int yy=0;
+        for(y=0;y<range;y++) {
+            d[yy] = s[yy];
+            yy+=width;
+        }
+        for(y=range;y<height-range;y++) {
+            int r=0;
+            int g=0;
+            int b=0;
+            int a=0;
+            int*f = weights;
+            int cy,cyy=yy-range*width;
+            for(cy=y-range;cy<y+range;cy++) {
+                r += s[cyy].r * f[0];
+                g += s[cyy].g * f[0];
+                b += s[cyy].b * f[0];
+                a += s[cyy].a * f[0];
+                cyy += width;
+                f++;
+            }
+            d[yy].r = r >> 16;
+            d[yy].g = g >> 16;
+            d[yy].b = b >> 16;
+            d[yy].a = a >> 16;
+            yy += width;
+        }
+        for(y=0;y<range;y++) {
+            d[yy] = s[yy];
+            yy += width;
+        }
+    }
+
+    free(tmp);
+    free(weights);
+    free(gauss);
+}
+
+
 RGBA* swf_ImageScale(RGBA*data, int width, int height, int newwidth, int newheight)
 {
     int x,y;
@@ -1346,6 +1488,14 @@ RGBA* swf_ImageScale(RGBA*data, int width, int height, int newwidth, int newheig
     if(swf_ImageGetNumberOfPaletteEntries2(data, width, height) == 2) {
 	monochrome=1;
 	encodeMonochromeImage(data, width, height, monochrome_colors);
+        int r1 = width / newwidth;
+        int r2 = height / newheight;
+        int r = r1<r2?r1:r2;
+        if(r>4) {
+            /* high-resolution monochrome images are usually dithered, so 
+               low-pass filter them first to get rid of any moire patterns */
+            blurImage(data, width, height, r+1);
+        }
     }
 
     tmpline = (rgba_int_t*)malloc(width*sizeof(rgba_int_t));
