@@ -23,6 +23,9 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #include <assert.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include "../rfxswf.h"
 
 /* one bit flag: */
 #define clip_type 0
@@ -105,7 +108,7 @@ static inline void add_pixel(RENDERBUF*dest, float x, int y, renderpoint_t*p)
 
 /* set this to 0.777777 or something if the "both fillstyles set while not inside shape"
    problem appears to often */
-#define CUT 0.5
+#define CUT 0.77887789
 
 #define INT(x) ((int)((x)+16)-16)
 
@@ -211,7 +214,7 @@ static void add_solidline(RENDERBUF*buf, double x1, double y1, double x2, double
         vy = (-dx/d);
     }
 
-    segments = width/2;
+    segments = (int)(width/2);
     if(segments < 2)
         segments = 2;
 
@@ -330,11 +333,11 @@ void swf_Render_AddImage(RENDERBUF*buf, U16 id, RGBA*img, int width, int height)
 {
     renderbuf_internal*i = (renderbuf_internal*)buf->internal;
 
-    bitmap_t*bm = rfx_calloc(sizeof(bitmap_t));
+    bitmap_t*bm = (bitmap_t*)rfx_calloc(sizeof(bitmap_t));
     bm->id = id;
     bm->width = width;
     bm->height = height;
-    bm->data = rfx_alloc(width*height*4);
+    bm->data = (RGBA*)rfx_alloc(width*height*4);
     memcpy(bm->data, img, width*height*4);
 
     bm->next = i->bitmaps;
@@ -362,7 +365,7 @@ void swf_Render_Delete(RENDERBUF*dest)
 
     /* delete line buffers */
     for(y=0;y<i->height2;y++) {
-        swf_DeleteTag(i->lines[y].points);
+        swf_DeleteTag(0, i->lines[y].points);
         i->lines[y].points = 0;
     }
 
@@ -380,7 +383,7 @@ void swf_Render_Delete(RENDERBUF*dest)
 
 static SHAPE2* linestyle2fillstyle(SHAPE2*shape)
 {
-    SHAPE2*s = rfx_calloc(sizeof(SHAPE2));
+    SHAPE2*s = (SHAPE2*)rfx_calloc(sizeof(SHAPE2));
     int t;
     s->numfillstyles = shape->numlinestyles;
     s->fillstyles = (FILLSTYLE*)rfx_calloc(sizeof(FILLSTYLE)*shape->numlinestyles);
@@ -482,7 +485,7 @@ void swf_RenderShape(RENDERBUF*dest, SHAPE2*shape, MATRIX*m, CXFORM*c, U16 _dept
             xx=x1;
 	    yy=y1;
 
-            parts = (int)(sqrt(c)/3);
+            parts = (int)(sqrt((float)c)/3);
             if(!parts) parts = 1;
 
             for(t=1;t<=parts;t++) {
@@ -566,6 +569,12 @@ static void fill_solid(RGBA*line, int*z, int y, int x1, int x2, RGBA col, U32 de
     }
 }
 
+static int inline clamp(int v)
+{
+    if(v>255) return 255;
+    else return v;
+}
+
 static void fill_bitmap(RGBA*line, int*z, int y, int x1, int x2, MATRIX*m, bitmap_t*b, int clipbitmap, U32 depth, double fmultiply)
 {
     int x = x1;
@@ -609,9 +618,90 @@ static void fill_bitmap(RGBA*line, int*z, int y, int x1, int x2, MATRIX*m, bitma
 	    col = b->data[yy*b->width+xx];
 	    ainv = 255-col.a;
 
-	    line[x].r = ((line[x].r*ainv)>>8)+col.r;
-	    line[x].g = ((line[x].g*ainv)>>8)+col.g;
-	    line[x].b = ((line[x].b*ainv)>>8)+col.b;
+	    line[x].r = clamp(((line[x].r*ainv)>>8)+col.r);
+	    line[x].g = clamp(((line[x].g*ainv)>>8)+col.g);
+	    line[x].b = clamp(((line[x].b*ainv)>>8)+col.b);
+	    line[x].a = 255;
+	    
+	    z[x] = depth;
+	}
+    } while(++x<x2);
+}
+
+static void fill_gradient(RGBA*line, int*z, int y, int x1, int x2, MATRIX*m, GRADIENT*g, int type, U32 depth, double fmultiply)
+{
+    int x = x1;
+    
+    double m11= m->sx*fmultiply/80, m21= m->r1*fmultiply/80;
+    double m12= m->r0*fmultiply/80, m22= m->sy*fmultiply/80;
+    double rx = m->tx*fmultiply/20.0;
+    double ry = m->ty*fmultiply/20.0;
+
+    double det = m11*m22 - m12*m21;
+    if(fabs(det) < 0.0005) { 
+	/* x direction equals y direction- the image is invisible */
+	return;
+    }
+    det = 1.0/det;
+
+    RGBA palette[512];
+    RGBA oldcol = g->rgba[0];
+    int r0 = g->ratios[0]*2;
+    int t;
+    for(t=0;t<r0;t++) 
+	palette[t] = oldcol;
+    for(t=1;t<g->num;t++) {
+	int r1 = g->ratios[t]*2;
+	RGBA newcol = g->rgba[t];
+	if(r0 == r1)
+	    continue;
+	//printf("%d %d->%d %02x%02x%02x%02x->%02x%02x%02x%02x\n", 
+	//	t, r0, r1, oldcol.r,oldcol.g,oldcol.b,oldcol.a,
+	//	newcol.r,newcol.g,newcol.b,newcol.a);
+	double f = 1.0 / (r1-r0);
+	double p0 = 1;
+	double p1 = 0;
+	int s;
+	for(;r0<=r1;r0++) {
+	    palette[r0].r = oldcol.r*p0 + newcol.r*p1;
+	    palette[r0].g = oldcol.g*p0 + newcol.g*p1;
+	    palette[r0].b = oldcol.b*p0 + newcol.b*p1;
+	    palette[r0].a = oldcol.a*p0 + newcol.a*p1;
+	    p0 -= f;
+	    p1 += f;
+	}
+	oldcol = newcol;
+    }
+    for(t=r0;t<512;t++) 
+	palette[t] = oldcol;
+
+    do {
+	if(depth >= z[x]) {
+	    RGBA col;
+	    double xx = (  (x - rx) * m22 - (y - ry) * m21)*det;
+	    double yy = (- (x - rx) * m12 + (y - ry) * m11)*det;
+	    int ainv;
+	    ainv = 255-col.a;
+
+	    if(type == FILL_LINEAR) {
+		int xr = xx*256;
+		if(xr<-256)
+		    xr = -256;
+		if(xr>255)
+		    xr = 255;
+		col = palette[xr+256];
+	    } else {
+		int xr = sqrt(xx*xx+yy*yy)*511;
+		if(xr<0)
+		    xr = 0;
+		if(xr>511)
+		    xr = 511;
+		col = palette[xr];
+	    }
+
+	    line[x].r = clamp(((line[x].r*ainv)>>8)+col.r);
+	    line[x].g = clamp(((line[x].g*ainv)>>8)+col.g);
+	    line[x].b = clamp(((line[x].b*ainv)>>8)+col.b);
 	    line[x].a = 255;
 	    
 	    z[x] = depth;
@@ -661,7 +751,7 @@ static void fill(RENDERBUF*dest, RGBA*line, int*zline, int y, int x1, int x2, st
 	    if(f->type == FILL_SOLID) {
                 /* plain color fill */
                 fill_solid(line, zline, y, x1, x2, f->color, l->p->depth);
-            } else if(f->type == FILL_TILED || f->type == FILL_CLIPPED) {
+            } else if(f->type == FILL_TILED || f->type == FILL_CLIPPED || f->type == (FILL_TILED|2) || f->type == (FILL_CLIPPED|2)) {
                 /* TODO: optimize (do this in add_pixel()?) */
                 bitmap_t* b = i->bitmaps;
                 while(b && b->id != f->id_bitmap) {
@@ -671,8 +761,10 @@ static void fill(RENDERBUF*dest, RGBA*line, int*zline, int y, int x1, int x2, st
                     fprintf(stderr, "Shape references unknown bitmap %d\n", f->id_bitmap);
                     fill_solid(line, zline, y, x1, x2, color_red, l->p->depth);
                 } else {
-                    fill_bitmap(line, zline, y, x1, x2, &f->m, b, FILL_CLIPPED?1:0, l->p->depth, i->multiply);
+                    fill_bitmap(line, zline, y, x1, x2, &f->m, b, /*clipped?*/f->type&1, l->p->depth, i->multiply);
                 }
+            } else if(f->type == FILL_LINEAR || f->type == FILL_RADIAL) {
+		fill_gradient(line, zline, y, x1, x2, &f->m, &f->gradient, f->type, l->p->depth, i->multiply);
             } else {
                 fprintf(stderr, "Undefined fillmode: %02x\n", f->type);
 	    }
@@ -777,12 +869,13 @@ static void change_state(int y, state_t* state, renderpoint_t*p)
                we always get (0,32), (32, 33), (33, 0) in the right order if
                they happen to fall on the same pixel.
                (not: (0,32), (33, 0), (32, 33))
+	       Notice: Weird fill styles appear if linestyles are involved, too.
             */
             fprintf(stderr, "<line %d: both fillstyles set while not inside shape>\n", y);
             return;
         }
         
-        n = rfx_calloc(sizeof(layer_t));
+        n = (layer_t*)rfx_calloc(sizeof(layer_t));
 
         if(DEBUG&2) printf("<+>");
 
@@ -836,6 +929,15 @@ void swf_Process(RENDERBUF*dest, U32 clipdepth)
 	state_t fillstate;
         memset(&fillstate, 0, sizeof(state_t));
         qsort(points, num, sizeof(renderpoint_t), compare_renderpoints);
+	/* resort points */
+	/*if(y==884) {
+	    for(n=0;n<num;n++) {
+		printf("%f (%d/%d) %d\n", points[n].x, 
+			points[n].shapeline->fillstyle0,
+			points[n].shapeline->fillstyle1,
+			points[n].shapeline->linestyle);
+	    }
+	}*/
 
 	if(i->lines[y].pending_clipdepth && !clipdepth) {
 	    fill_clip(line, zline, y, 0, i->width2, i->lines[y].pending_clipdepth);
@@ -845,8 +947,8 @@ void swf_Process(RENDERBUF*dest, U32 clipdepth)
         for(n=0;n<num;n++) {
             renderpoint_t*p = &points[n];
             renderpoint_t*next= n<num-1?&points[n+1]:0;
-            int startx = p->x;
-            int endx = next?next->x:i->width2;
+            int startx = (int)p->x;
+            int endx = (int)(next?next->x:i->width2);
             if(endx > i->width2)
                 endx = i->width2;
             if(startx < 0)
@@ -946,11 +1048,12 @@ typedef struct
     SHAPE2**glyphs;
 } font_t;
 
+enum CHARACTER_TYPE {none_type, shape_type, image_type, text_type, edittext_type, font_type, sprite_type};
 typedef struct
 {
     TAG*tag;
     SRECT*bbox;
-    enum {none_type, shape_type, image_type, text_type, font_type} type;
+    enum CHARACTER_TYPE type;
     union {
         SHAPE2*shape;
         font_t*font;
@@ -1051,29 +1154,110 @@ static void textcallback(void*self, int*chars, int*xpos, int nr, int fontid, int
     }
 }
 
-void swf_RenderSWF(RENDERBUF*buf, SWF*swf)
+static void renderFromTag(RENDERBUF*buf, character_t*idtable, TAG*firstTag, MATRIX*m)
 {
-    TAG*tag;
-    int t;
-    int numplacements;
-    RGBA color;
+    TAG*tag = 0;
+    int numplacements = 0;
     SWFPLACEOBJECT* placements;
-    
-    character_t* idtable = rfx_calloc(sizeof(character_t)*65536);            // id to character mapping
-    SWFPLACEOBJECT** depthtable = rfx_calloc(sizeof(SWFPLACEOBJECT*)*65536); // depth to placeobject mapping
-    
-    tag = swf->firstTag;
+
+    tag = firstTag;
     numplacements = 0;
     while(tag) {
         if(tag->id == ST_PLACEOBJECT || 
            tag->id == ST_PLACEOBJECT2) {
 	    numplacements++;
 	}
+	if(tag->id == ST_SHOWFRAME || tag->id == ST_END)
+	    break;
 	tag = tag->next;
     }
-    placements = rfx_calloc(sizeof(SWFPLACEOBJECT)*numplacements);
+    placements = (SWFPLACEOBJECT*)rfx_calloc(sizeof(SWFPLACEOBJECT)*numplacements);
     numplacements = 0;
 
+    tag = firstTag;
+    while(tag) {
+	if(swf_isPlaceTag(tag)) {
+	    SWFPLACEOBJECT p;
+	    swf_GetPlaceObject(tag, &p);
+	    /* TODO: add move and deletion */
+	    placements[numplacements++] = p;
+	    swf_PlaceObjectFree(&p); //dirty! but it only frees fields we don't use
+	}
+	if(tag->id == ST_SHOWFRAME || tag->id == ST_END)
+	    break;
+        tag = tag->next;
+    }
+
+    qsort(placements, numplacements, sizeof(SWFPLACEOBJECT), compare_placements);
+     
+    int t;
+    for(t=0;t<numplacements;t++) {
+        SWFPLACEOBJECT*p = &placements[t];
+        int id = p->id;
+	MATRIX m2;
+	swf_MatrixJoin(&m2, m, &p->matrix);
+            
+        if(!idtable[id].tag) { 
+            fprintf(stderr, "rfxswf: Id %d is unknown\n", id);
+            continue;
+        }
+
+        if(idtable[id].type == shape_type) {
+            //SRECT sbbox = swf_TurnRect(*idtable[id].bbox, &p->matrix);
+            swf_RenderShape(buf, idtable[id].obj.shape, &m2, &p->cxform, p->depth, p->clipdepth);
+	} else if(idtable[id].type == sprite_type) {
+	    swf_UnFoldSprite(idtable[id].tag);
+	    renderFromTag(buf, idtable, idtable[id].tag->next, &m2);
+	    swf_FoldSprite(idtable[id].tag);
+        } else if(idtable[id].type == text_type) {
+	    TAG* tag = idtable[id].tag;
+	    textcallbackblock_t info;
+	    MATRIX mt;
+
+	    swf_SetTagPos(tag, 0);
+	    swf_GetU16(tag);
+	    swf_GetRect(tag,0);
+  	    swf_GetMatrix(tag,&mt);
+	    swf_MatrixJoin(&info.m, &m2, &mt);
+	    /*printf("Text matrix:\n");
+	    swf_DumpMatrix(stdout, &m);
+	    printf("Placement matrix:\n");
+	    swf_DumpMatrix(stdout, &p->matrix);
+	    printf("Final matrix:\n");
+	    swf_DumpMatrix(stdout, &info.m);*/
+
+	    info.idtable = idtable;
+	    info.depth = p->depth;
+	    info.cxform = &p->cxform;
+	    info.clipdepth = p->clipdepth;
+	    info.buf = buf;
+	    
+	    swf_ParseDefineText(tag, textcallback, &info);
+        } else if(idtable[id].type == edittext_type) {
+	    TAG* tag = idtable[id].tag;
+            U16 flags = swf_GetBits(tag, 16);
+	    if(flags & ET_HASTEXT) {
+                fprintf(stderr, "edittext not supported yet (id %d)\n", id);
+            }
+        } else {
+            fprintf(stderr, "Unknown/Unsupported Object Type for id %d: %s\n", id, swf_TagGetName(idtable[id].tag));
+        }
+    }
+
+    free(placements);
+}
+
+void swf_RenderSWF(RENDERBUF*buf, SWF*swf)
+{
+    TAG*tag;
+    int t;
+    RGBA color;
+
+    swf_OptimizeTagOrder(swf);
+    swf_FoldAll(swf);
+    
+    character_t* idtable = (character_t*)rfx_calloc(sizeof(character_t)*65536);            // id to character mapping
+    
     /* set background color */
     color = swf_GetSWFBackgroundColor(swf);
     swf_Render_SetBackgroundColor(buf, color);
@@ -1084,11 +1268,11 @@ void swf_RenderSWF(RENDERBUF*buf, SWF*swf)
         if(swf_isDefiningTag(tag)) {
             int id = swf_GetDefineID(tag);
             idtable[id].tag = tag;
-            idtable[id].bbox = rfx_alloc(sizeof(SRECT));
+            idtable[id].bbox = (SRECT*)rfx_alloc(sizeof(SRECT));
             *idtable[id].bbox = swf_GetDefineBBox(tag);
 
             if(swf_isShapeTag(tag)) {
-                SHAPE2* shape = rfx_calloc(sizeof(SHAPE2));
+                SHAPE2* shape = (SHAPE2*)rfx_calloc(sizeof(SHAPE2));
                 swf_ParseDefineShape(tag, shape);
                 idtable[id].type = shape_type;
                 idtable[id].obj.shape = shape;
@@ -1102,11 +1286,11 @@ void swf_RenderSWF(RENDERBUF*buf, SWF*swf)
                       tag->id == ST_DEFINEFONT2) {
 		int t;
 		SWFFONT*swffont;
-		font_t*font = rfx_calloc(sizeof(font_t));
+		font_t*font = (font_t*)rfx_calloc(sizeof(font_t));
 		idtable[id].obj.font = font;
                 swf_FontExtract(swf,id,&swffont);
 		font->numchars = swffont->numchars;
-		font->glyphs = rfx_calloc(sizeof(SHAPE2*)*font->numchars);
+		font->glyphs = (SHAPE2**)rfx_calloc(sizeof(SHAPE2*)*font->numchars);
 		for(t=0;t<font->numchars;t++) {
 		    if(!swffont->glyph[t].shape->fillstyle.n) {
 			/* the actual fill color will be overwritten while rendering */
@@ -1123,61 +1307,18 @@ void swf_RenderSWF(RENDERBUF*buf, SWF*swf)
             } else if(tag->id == ST_DEFINETEXT ||
                       tag->id == ST_DEFINETEXT2) {
                 idtable[id].type = text_type;
-            }
-        } else if(tag->id == ST_PLACEOBJECT || 
-                  tag->id == ST_PLACEOBJECT2) {
-            SWFPLACEOBJECT p;
-            swf_GetPlaceObject(tag, &p);
-            /* TODO: add move and deletion */
-            placements[numplacements++] = p;
-	    swf_PlaceObjectFree(&p); //dirty! but it only frees fields we don't use
+            } else if(tag->id == ST_DEFINESPRITE) {
+		idtable[id].type = sprite_type;
+            } else if(tag->id == ST_DEFINEEDITTEXT) {
+		idtable[id].type = edittext_type;
+	    }
         }
-        tag = tag->next;
+	tag = tag->next;
     }
-
-    qsort(placements, numplacements, sizeof(SWFPLACEOBJECT), compare_placements);
-      
-    for(t=0;t<numplacements;t++) {
-        SWFPLACEOBJECT*p = &placements[t];
-        int id = p->id;
-            
-        if(!idtable[id].tag) { 
-            fprintf(stderr, "rfxswf: Id %d is unknown\n", id);
-            continue;
-        }
-
-        if(idtable[id].type == shape_type) {
-            //SRECT sbbox = swf_TurnRect(*idtable[id].bbox, &p->matrix);
-            swf_RenderShape(buf, idtable[id].obj.shape, &p->matrix, &p->cxform, p->depth, p->clipdepth);
-        } else if(idtable[id].type == text_type) {
-	    TAG* tag = idtable[id].tag;
-	    textcallbackblock_t info;
-	    MATRIX m;
-
-	    swf_SetTagPos(tag, 0);
-	    swf_GetU16(tag);
-	    swf_GetRect(tag,0);
-  	    swf_GetMatrix(tag,&m);
-	    swf_MatrixJoin(&info.m, &p->matrix, &m);
-	    /*printf("Text matrix:\n");
-	    swf_DumpMatrix(stdout, &m);
-	    printf("Placement matrix:\n");
-	    swf_DumpMatrix(stdout, &p->matrix);
-	    printf("Final matrix:\n");
-	    swf_DumpMatrix(stdout, &info.m);*/
-
-	    info.idtable = idtable;
-	    info.depth = p->depth;
-	    info.cxform = &p->cxform;
-	    info.clipdepth = p->clipdepth;
-	    info.buf = buf;
-	    
-	    swf_ParseDefineText(tag, textcallback, &info);
-        } else {
-            fprintf(stderr, "Unknown/Unsupported Object Type for id %d: %s\n", id, swf_TagGetName(idtable[id].tag));
-        }
-    }
-
+    MATRIX m;
+    swf_GetMatrix(0, &m);
+    renderFromTag(buf, idtable, swf->firstTag, &m);
+    
     /* free id and depth tables again */
     for(t=0;t<65536;t++) {
         if(idtable[t].bbox) {
@@ -1207,8 +1348,6 @@ void swf_RenderSWF(RENDERBUF*buf, SWF*swf)
 	    }
 	}
     }
-    free(placements);
     free(idtable);
-    free(depthtable);
 }
-
+    
