@@ -23,12 +23,14 @@
 
 // Matrix & Math tools for SWF files
 
+#include "../rfxswf.h"
+
 #define S64 long long
 SFIXED RFXSWF_SP(SFIXED a1,SFIXED a2,SFIXED b1,SFIXED b2)
 { S64 a = ((S64)a1*(S64)b1+(S64)a2*(S64)b2)>>16;
   SFIXED result = (SFIXED)(a);
   if(a!=result) 
-      fprintf(stderr, "Warning: overflow in matrix multiplication");
+      fprintf(stderr, "Warning: overflow in matrix multiplication\n");
   return result;
 }
 SFIXED RFXSWF_QFIX(int zaehler,int nenner) // bildet Quotient von zwei INTs in SFIXED
@@ -100,6 +102,7 @@ U16 swf_GetDefineID(TAG * t)
     case ST_DEFINESHAPE3:
     case ST_DEFINESHAPE4:
     case ST_DEFINEMORPHSHAPE:
+    case ST_DEFINEMORPHSHAPE2:
     case ST_DEFINEEDITTEXT:
     case ST_DEFINEBITS:
     case ST_DEFINEBITSJPEG2:
@@ -118,7 +121,9 @@ U16 swf_GetDefineID(TAG * t)
     case ST_DEFINEFONTINFO: //pseudodefine
     case ST_DEFINEFONTINFO2: //pseudodefine
     case ST_DEFINEFONTALIGNZONES: //pseudodefine
+    case ST_DEFINEFONTNAME: //pseudodefine
     case ST_DEFINETEXT:
+    case ST_DEFINEBINARY:
     case ST_DEFINETEXT2:
     case ST_DEFINESOUND:
     case ST_DEFINESPRITE:
@@ -155,6 +160,7 @@ SRECT swf_GetDefineBBox(TAG * t)
   { case ST_DEFINESHAPE:
     case ST_DEFINESHAPE2:
     case ST_DEFINESHAPE3:
+    case ST_DEFINESHAPE4:
     case ST_DEFINEEDITTEXT:
     case ST_DEFINETEXT:
     case ST_DEFINETEXT2:
@@ -223,6 +229,7 @@ static int swf_definingtagids[] =
  ST_DEFINESHAPE3,
  ST_DEFINESHAPE4,
  ST_DEFINEMORPHSHAPE,
+ ST_DEFINEMORPHSHAPE2,
  ST_DEFINEFONT,
  ST_DEFINEFONT2,
  ST_DEFINEFONT3,
@@ -240,6 +247,7 @@ static int swf_definingtagids[] =
  ST_DEFINEBUTTON2,
  ST_DEFINESOUND,
  ST_DEFINEVIDEOSTREAM,
+ ST_DEFINEBINARY,
  -1
 };
 
@@ -252,6 +260,7 @@ static int swf_spritetagids[] =
  ST_REMOVEOBJECT,
  ST_REMOVEOBJECT2,
  ST_DOACTION,
+ ST_DOABC,
  ST_STARTSOUND,
  ST_FRAMELABEL,
  ST_SOUNDSTREAMHEAD,
@@ -267,6 +276,7 @@ static int swf_pseudodefiningtagids[] =
  ST_DEFINEFONTINFO,
  ST_DEFINEFONTINFO2,
  ST_DEFINEFONTALIGNZONES,
+ ST_DEFINEFONTNAME,
  ST_DEFINEBUTTONCXFORM,
  ST_DEFINEBUTTONSOUND,
  ST_DEFINESCALINGGRID,
@@ -383,7 +393,7 @@ char* swf_GetName(TAG * t)
     switch(swf_GetTagID(t))
     {
         case ST_FRAMELABEL:
-            name = &t->data[swf_GetTagPos(t)];
+            name = (char*)&t->data[swf_GetTagPos(t)];
         break;
         case ST_PLACEOBJECT3:
         case ST_PLACEOBJECT2: {   
@@ -403,7 +413,7 @@ char* swf_GetName(TAG * t)
               swf_GetU16(t);
             if(flags&PF_NAME) {
               swf_ResetReadBits(t);
-              name = &t->data[swf_GetTagPos(t)];
+              name = (char*)&t->data[swf_GetTagPos(t)];
             }
         }
         break;
@@ -422,13 +432,13 @@ void swf_GetMorphGradient(TAG * tag, GRADIENT * gradient1, GRADIENT * gradient2)
     
     if(gradient1) {
 	gradient1->num = num;
-	gradient1->rgba = rfx_calloc(sizeof(RGBA)*gradient1->num);
-	gradient1->ratios = rfx_calloc(sizeof(gradient1->ratios[0])*gradient1->num);
+	gradient1->rgba = (RGBA*)rfx_calloc(sizeof(RGBA)*gradient1->num);
+	gradient1->ratios = (U8*)rfx_calloc(sizeof(gradient1->ratios[0])*gradient1->num);
     }
     if(gradient2) {
 	gradient2->num = num;
-	gradient2->rgba = rfx_calloc(sizeof(RGBA)*gradient2->num);
-	gradient2->ratios = rfx_calloc(sizeof(gradient2->ratios[0])*gradient2->num);
+	gradient2->rgba = (RGBA*)rfx_calloc(sizeof(RGBA)*gradient2->num);
+	gradient2->ratios = (U8*)rfx_calloc(sizeof(gradient2->ratios[0])*gradient2->num);
     }
     for(t=0;t<num;t++)
     {
@@ -454,6 +464,92 @@ void swf_GetMorphGradient(TAG * tag, GRADIENT * gradient1, GRADIENT * gradient2)
 #define DEBUG_ENUMERATE if(0)
 //#define DEBUG_ENUMERATE
 
+void enumerateUsedIDs_fillstyle(TAG * tag, int t, void (*callback)(TAG*, int, void*), void*callback_data, int num, int morph)
+{
+    int type;
+    type = swf_GetU8(tag); //type
+    DEBUG_ENUMERATE printf("fill style %d) type=%02x (tagpos=%d)\n", t, type, tag->pos);
+    if(type == 0) {
+	RGBA color;
+	if(num >= 3)
+	    {swf_GetRGBA(tag, &color);if(morph) swf_GetRGBA(tag, NULL);}
+	else 
+	    {swf_GetRGB(tag, &color);if(morph) swf_GetRGB(tag, NULL);}
+	DEBUG_ENUMERATE printf("               %02x%02x%02x%02x\n", color.r,color.g,color.b,color.a);
+    }
+    else if(type == 0x10 || type == 0x12 || type == 0x13)
+    {
+	swf_ResetReadBits(tag);
+	MATRIX m;
+	swf_GetMatrix(tag, &m);
+	DEBUG_ENUMERATE swf_DumpMatrix(stdout, &m);
+	if(morph) {
+	    swf_GetMatrix(tag, &m);
+	    DEBUG_ENUMERATE swf_DumpMatrix(stdout, &m);
+	}
+	swf_ResetReadBits(tag);
+	if(morph) {
+	    swf_GetMorphGradient(tag, NULL, NULL);
+	    if(type == 0x13) {
+		swf_GetU16(tag);
+		swf_GetU16(tag);
+	    }
+	} else {
+	    GRADIENT g;
+	    swf_GetGradient(tag, &g, /*alpha*/ num>=3?1:0);
+	    DEBUG_ENUMERATE swf_DumpGradient(stdout, &g);
+	    if(type == 0x13)
+		swf_GetU16(tag);
+	}
+    }
+    else if(type == 0x40 || type == 0x41 || type == 0x42 || type == 0x43)
+    {
+	swf_ResetReadBits(tag);
+	if(tag->data[tag->pos] != 0xff ||
+	   tag->data[tag->pos+1] != 0xff)
+	(callback)(tag, tag->pos, callback_data);
+
+	swf_GetU16(tag);
+	swf_ResetReadBits(tag);
+	swf_GetMatrix(tag, NULL);
+	if(morph)
+	    swf_GetMatrix(tag, NULL);
+    }
+    else {
+	fprintf(stderr, "rfxswf:swftools.c Unknown fillstyle:0x%02x in tag %02d\n",type, tag->id);
+    }
+}
+
+void enumerateUsedIDs_linestyle(TAG * tag, int t, void (*callback)(TAG*, int, void*), void*callback_data, int num, int morph)
+{
+    U16  width;
+    RGBA color;
+    width = swf_GetU16(tag);
+    char fill=0;
+    if(morph)
+	swf_GetU16(tag);
+    if(num >= 4) {
+	U16 flags = swf_GetU16(tag);
+	DEBUG_ENUMERATE printf("line style %d) flags: %08x\n", t, flags);
+	if((flags & 0x30) == 0x20) {
+	    U16 miter = swf_GetU16(tag); // miter limit
+	    DEBUG_ENUMERATE printf("line style %d) miter join: %08x\n", t, miter);
+	}
+	if(flags & 0x08) {
+	    fill = 1;
+	}
+    }
+    if(!fill) {
+	if(num >= 3)
+	    {swf_GetRGBA(tag, &color);if(morph) swf_GetRGBA(tag, NULL);}
+	else
+	    {swf_GetRGB(tag, &color);if(morph) swf_GetRGB(tag, NULL);}
+    } else {
+	enumerateUsedIDs_fillstyle(tag, t, callback, callback_data, num, morph);
+    }
+    DEBUG_ENUMERATE printf("line style %d) width=%.2f color=%02x%02x%02x%02x \n", t, width/20.0, color.r,color.g,color.b,color.a);
+}
+
 void enumerateUsedIDs_styles(TAG * tag, void (*callback)(TAG*, int, void*), void*callback_data, int num, int morph)
 {
     U16 count;
@@ -465,51 +561,7 @@ void enumerateUsedIDs_styles(TAG * tag, void (*callback)(TAG*, int, void*), void
     DEBUG_ENUMERATE printf("%d fill styles\n", count);
     for(t=0;t<count;t++)
     {
-	int type;
-	U8*pos;
-	type = swf_GetU8(tag); //type
-	DEBUG_ENUMERATE printf("fill style %d) %02x (tagpos=%d)\n", t, type, tag->pos);
-	if(type == 0) {
-	    if(num >= 3)
-		{swf_GetRGBA(tag, NULL);if(morph) swf_GetRGBA(tag, NULL);}
-	    else 
-		{swf_GetRGB(tag, NULL);if(morph) swf_GetRGB(tag, NULL);}
-	}
-	else if(type == 0x10 || type == 0x12 || type == 0x13)
-	{
-	    swf_ResetReadBits(tag);
-	    MATRIX m;
-	    swf_GetMatrix(tag, &m);
-	    DEBUG_ENUMERATE swf_DumpMatrix(stdout, &m);
-	    if(morph)
-		swf_GetMatrix(tag, NULL);
-	    swf_ResetReadBits(tag);
-	    if(morph)
-		swf_GetMorphGradient(tag, NULL, NULL);
-	    else {
-		GRADIENT g;
-		swf_GetGradient(tag, &g, /*alpha*/ num>=3?1:0);
-		DEBUG_ENUMERATE swf_DumpGradient(stdout, &g);
-		if(type == 0x13)
-		    swf_GetU16(tag);
-	    }
-	}
-	else if(type == 0x40 || type == 0x41 || type == 0x42 || type == 0x43)
-	{
-	    swf_ResetReadBits(tag);
-	    if(tag->data[tag->pos] != 0xff ||
-	       tag->data[tag->pos+1] != 0xff)
-	    (callback)(tag, tag->pos, callback_data);
-
-	    swf_GetU16(tag);
-	    swf_ResetReadBits(tag);
-	    swf_GetMatrix(tag, NULL);
-	    if(morph)
-		swf_GetMatrix(tag, NULL);
-	}
-	else {
-	    fprintf(stderr, "rfxswf:swftools.c Unknown fillstyle:0x%02x in tag %02x\n",type, tag->id);
-	}
+	enumerateUsedIDs_fillstyle(tag, t, callback, callback_data, num, morph);
     }
     swf_ResetReadBits(tag);
     count = swf_GetU8(tag); // line style array
@@ -518,24 +570,7 @@ void enumerateUsedIDs_styles(TAG * tag, void (*callback)(TAG*, int, void*), void
     DEBUG_ENUMERATE printf("%d line styles\n", count);
     for(t=0;t<count;t++) 
     {
-	U16  width;
-	RGBA color;
-	width = swf_GetU16(tag);
-	if(morph)
-	    swf_GetU16(tag);
-	if(num >= 4) {
-	    U16 flags = swf_GetU16(tag);
-	    if(flags & 0x2000)
-		swf_GetU16(tag); // miter limit
-	    if(flags & 0x0800) {
-		fprintf(stderr, "Filled strokes parsing not yet supported\n");
-	    }
-	}
-	if(num >= 3)
-	    {swf_GetRGBA(tag, &color);if(morph) swf_GetRGBA(tag, NULL);}
-	else
-	    {swf_GetRGB(tag, &color);if(morph) swf_GetRGB(tag, NULL);}
-	DEBUG_ENUMERATE printf("line style %d: %02x%02x%02x%02x \n", t, color.r,color.g,color.b,color.a);
+	enumerateUsedIDs_linestyle(tag, t, callback, callback_data, num, morph);
     }
 }
 
@@ -601,8 +636,14 @@ void enumerateUsedIDs(TAG * tag, int base, void (*callback)(TAG*, int, void*), v
 	    }
 	} break;
 
+        case ST_DOABC:
+        case ST_RAWABC:
+        break;
+
 	case ST_FREECHARACTER: /* unusual tags, which all start with an ID */
 	case ST_NAMECHARACTER:
+	case ST_DEFINEBINARY:
+	case ST_DEFINEFONTNAME:
 	case ST_GENERATORTEXT:
 	    callback(tag, tag->pos + base, callback_data);
         break;
@@ -645,12 +686,12 @@ void enumerateUsedIDs(TAG * tag, int base, void (*callback)(TAG*, int, void*), v
 		if(id == ST_END)
 		    break;
 		tag2->len = tag2->memsize = len;
-		tag2->data = rfx_alloc(len);
+		tag2->data = (U8*)rfx_alloc(len);
 		memcpy(tag2->data, &tag->data[tag->pos], len);
 		/* I never saw recursive sprites, but they are (theoretically) 
 		   possible, so better add base here again */
 		enumerateUsedIDs(tag2, tag->pos + base, callback, callback_data);
-		swf_DeleteTag(tag2);
+		swf_DeleteTag(0, tag2);
 		swf_GetBlock(tag, NULL, len);
 	    }
 	} 
@@ -668,8 +709,8 @@ void enumerateUsedIDs(TAG * tag, int base, void (*callback)(TAG*, int, void*), v
 	    }
 	    while(1)
 	    {
-		U16 charid;
-		if(!swf_GetU8(tag)) //flags
+		U8 flags = swf_GetU8(tag);
+		if(!flags) //flags
 		    break; 
 		callback(tag, tag->pos + base, callback_data);
 		swf_GetU16(tag); //char
@@ -680,6 +721,16 @@ void enumerateUsedIDs(TAG * tag, int base, void (*callback)(TAG*, int, void*), v
 		  swf_ResetReadBits(tag);
 		  swf_GetCXForm(tag, NULL, 1);
 		}
+		if(flags&0x10) {
+		    U8 num = swf_GetU8(tag);
+		    int t;
+		    for(t=0;t<num;t++) {
+			swf_DeleteFilter(swf_GetFilter(tag));
+		    }
+		}
+		if(flags&0x20) {
+		    U8 blendmode = swf_GetU8(tag);
+		}
 	    }
 	    // ...
 	}
@@ -817,6 +868,7 @@ void enumerateUsedIDs(TAG * tag, int base, void (*callback)(TAG*, int, void*), v
 		linebits = swf_GetBits(tag, 4);
 		DEBUG_ENUMERATE printf("fillbits=%d linebits=%d\n", fillbits, linebits);
 		swf_ResetReadBits(tag);
+		int x=0,y=0;
 		while(1) {
 		    int flags;
 		    flags = swf_GetBits(tag, 1);
@@ -826,10 +878,9 @@ void enumerateUsedIDs(TAG * tag, int base, void (*callback)(TAG*, int, void*), v
 			    break;
 			if(flags&1) { //move
 			    int n = swf_GetBits(tag, 5); 
-			    int x,y;
 			    x = swf_GetBits(tag, n); //x
 			    y = swf_GetBits(tag, n); //y
-			    DEBUG_ENUMERATE printf("move %f %f\n",x/20.0,y/20.0);
+			    DEBUG_ENUMERATE printf("moveTo %.2f %.2f\n",x/20.0,y/20.0);
 			}
 			if(flags&2) { //fill0
 			    int fill0;
@@ -857,15 +908,18 @@ void enumerateUsedIDs(TAG * tag, int base, void (*callback)(TAG*, int, void*), v
 			if(flags) { //straight edge
 			    int n = swf_GetBits(tag, 4) + 2;
 			    if(swf_GetBits(tag, 1)) { //line flag
-				int x,y;
-				x = swf_GetSBits(tag, n); //delta x
-				y = swf_GetSBits(tag, n); //delta y
-				DEBUG_ENUMERATE printf("line %f %f\n",x/20.0,y/20.0);
+				x += swf_GetSBits(tag, n); //delta x
+				y += swf_GetSBits(tag, n); //delta y
+				DEBUG_ENUMERATE printf("lineTo %.2f %.2f\n",x/20.0,y/20.0);
 			    } else {
 				int v=swf_GetBits(tag, 1);
 				int d;
 				d = swf_GetSBits(tag, n); //vert/horz
-				DEBUG_ENUMERATE printf("%s %f\n",v?"vertical":"horizontal", d/20.0);
+				if(!v)
+				    x += d;
+				else
+				    y += d;
+				DEBUG_ENUMERATE printf("lineTo %.2f %.2f (%s)\n",x/20.0,y/20.0, v?"vertical":"horizontal");
 			    }
 			} else { //curved edge
 			    int n = swf_GetBits(tag, 4) + 2;
@@ -874,7 +928,7 @@ void enumerateUsedIDs(TAG * tag, int base, void (*callback)(TAG*, int, void*), v
 			    y1 = swf_GetSBits(tag, n);
 			    x2 = swf_GetSBits(tag, n);
 			    y2 = swf_GetSBits(tag, n);
-			    DEBUG_ENUMERATE printf("curve %f %f %f %f\n", x1/20.0, y1/20.0, x2/20.0, y2/20.0);
+			    DEBUG_ENUMERATE printf("splineTo %.2f %.2f %.2f %.2f\n", x1/20.0, y1/20.0, x2/20.0, y2/20.0);
 			}
 		    }
 		}
@@ -912,12 +966,13 @@ void swf_GetUsedIDs(TAG * t, int * positions)
     enumerateUsedIDs(t, 0, callbackFillin, &ptr);
 }
 
-void swf_Relocate (SWF*swf, char*bitmap)
+char swf_Relocate (SWF*swf, char*bitmap)
 {
     TAG*tag;
     int slaveids[65536];
     memset(slaveids, -1, sizeof(slaveids));
     tag = swf->firstTag;
+    char ok = 1;
     while(tag)
     {
 	int num; 
@@ -933,8 +988,7 @@ void swf_Relocate (SWF*swf, char*bitmap)
 
 	    if(!bitmap[id]) { //free
 		newid = id;
-	    }
-	    else {
+	    } else {
 		newid = 0;
 		for (t=1;t<65536;t++)
 		{
@@ -944,6 +998,10 @@ void swf_Relocate (SWF*swf, char*bitmap)
 			break;
 		    }
 		}
+                if(t==65536) {
+                    fprintf(stderr, "swf_Relocate: Couldn't relocate: Out of IDs\n");
+                    return 0;
+                }
 	    }
 	    bitmap[newid] = 1;
 	    slaveids[id] = newid;
@@ -953,7 +1011,7 @@ void swf_Relocate (SWF*swf, char*bitmap)
 	
 	num = swf_GetNumUsedIDs(tag);
 	if(num) {
-	    ptr = rfx_alloc(sizeof(int)*num);
+	    ptr = (int*)rfx_alloc(sizeof(int)*num);
 	    swf_GetUsedIDs(tag, ptr);
 
 	    for(t=0;t<num;t++) {
@@ -961,14 +1019,17 @@ void swf_Relocate (SWF*swf, char*bitmap)
 		if(slaveids[id]<0) {
 		    fprintf(stderr, "swf_Relocate: Mapping id (%d) never encountered before in %s\n", id,
 			    swf_TagGetName(tag));
-		    return ;
+                    ok = 0;
+		} else {
+		    id = slaveids[id];
+		    PUT16(&tag->data[ptr[t]], id);
 		}
-		id = slaveids[id];
-		PUT16(&tag->data[ptr[t]], id);
 	    }
+            free(ptr);
 	}
 	tag=tag->next;
     }
+    return ok;
 }
 
 /* untested */
@@ -988,7 +1049,7 @@ void swf_Relocate2(SWF*swf, int*id2id)
 	if(num) {
 	    int *ptr;
 	    int t;
-	    ptr = rfx_alloc(sizeof(int)*num);
+	    ptr = (int*)rfx_alloc(sizeof(int)*num);
 	    swf_GetUsedIDs(tag, ptr);
 	    for(t=0;t<num;t++) {
 		int id = GET16(&tag->data[ptr[t]]);
@@ -997,6 +1058,7 @@ void swf_Relocate2(SWF*swf, int*id2id)
 		    PUT16(&tag->data[ptr[t]], id);
 		}
 	    }
+            free(ptr);
 	}
     }
 }
@@ -1154,10 +1216,10 @@ static int tagHash(TAG*tag)
 void swf_Optimize(SWF*swf)
 {
     const int hash_size = 131072;
-    char* dontremap = rfx_calloc(sizeof(char)*65536);
-    U16* remap = rfx_alloc(sizeof(U16)*65536);
-    TAG* id2tag = rfx_calloc(sizeof(TAG*)*65536);
-    TAG** hashmap = rfx_calloc(sizeof(TAG*)*hash_size);
+    char* dontremap = (char*)rfx_calloc(sizeof(char)*65536);
+    U16* remap = (U16*)rfx_alloc(sizeof(U16)*65536);
+    TAG* id2tag = (TAG*)rfx_calloc(sizeof(TAG*)*65536);
+    TAG** hashmap = (TAG**)rfx_calloc(sizeof(TAG*)*hash_size);
     TAG* tag;
     int t;
     for(t=0;t<65536;t++) {
@@ -1185,7 +1247,7 @@ void swf_Optimize(SWF*swf)
 
         /* remap the tag */
         int num = swf_GetNumUsedIDs(tag);
-        int*positions = rfx_alloc(sizeof(int)*num);
+        int*positions = (int*)rfx_alloc(sizeof(int)*num);
         int t;
         swf_GetUsedIDs(tag, positions);
         for(t=0;t<num;t++) {
@@ -1219,9 +1281,7 @@ void swf_Optimize(SWF*swf)
 		/* we found two identical tags- remap one
 		   of them */
                 remap[id] = swf_GetDefineID(tag2);
-                swf_DeleteTag(tag);
-                if(tag == swf->firstTag)
-                    swf->firstTag = next;
+                swf_DeleteTag(swf, tag);
             }
         } else if(swf_isPseudoDefiningTag(tag)) {
             int id = swf_GetDefineID(tag);
@@ -1229,9 +1289,7 @@ void swf_Optimize(SWF*swf)
                 /* if this tag was remapped, we don't
                    need the helper tag anymore. Discard
                    it. */
-                swf_DeleteTag(tag);
-                if(tag == swf->firstTag)
-                    swf->firstTag = next;
+                swf_DeleteTag(swf, tag);
             }
         }
 
@@ -1266,7 +1324,7 @@ void swf_SetDefineBBox(TAG * tag, SRECT newbbox)
 	      swf_ResetReadBits(tag);
 	      after_bbox_offset = tag->pos;
 	      len = tag->len - after_bbox_offset;
-	      data = malloc(len);
+	      data = (U8*)malloc(len);
 	      memcpy(data, &tag->data[after_bbox_offset], len);
 	      tag->writeBit = 0;
 	      tag->len = 2;
