@@ -29,16 +29,25 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 extern "C" {
 #endif
 
+extern char path_seperator;
+
 #ifdef WIN32
 char* getRegistryEntry(char*path);
 int setRegistryEntry(char*key,char*value);
 #endif
 
-char* getInstallationPath();
-char* concatPaths(char*base, char*add);
-char* stripFilename(char*filename, char*newext);
+typedef struct _memfile {
+    void*data;
+    int len;
+} memfile_t;
+memfile_t* memfile_open(const char*path);
+void memfile_close(memfile_t*file);
 
-char* mktempname(char*);
+char* getInstallationPath();
+char* concatPaths(const char*base, const char*add);
+char* stripFilename(const char*filename, const char*newext);
+
+char* mktempname(char*buffer);
 
 #ifdef __cplusplus
 }
