@@ -26,12 +26,6 @@
 
 #include "mem.h"
 #include "rfxswf.h"
-
-#ifdef HAVE_JPEGLIB
-#define HAVE_BOOLEAN
-#include <jpeglib.h>
-#endif // HAVE_JPEGLIB
-
 #ifdef HAVE_ZLIB
 #include <zlib.h>
 #endif // HAVE_ZLIB
@@ -46,8 +40,13 @@
 #include <time.h>
 #endif
 
+#ifdef HAVE_IO_H
+#include <io.h>
+#endif
+
 #include "./bitio.h"
 #include "./MD5.h"
+#include "./os.h"
 
 // internal constants
 
@@ -65,11 +64,6 @@ U32   swf_GetTagLen(TAG * t) { return t->len; }
 U8*   swf_GetTagLenPtr(TAG * t) { return &(t->data[t->len]); }
 U32   swf_GetTagPos(TAG * t)   { return t->pos; }
 
-// for future purpose: avoid high level lib functions to change tagpos/bitpos
-
-#define swf_SaveTagPos(tag)
-#define swf_RestoreTagPos(tag)
-
 void swf_SetTagPos(TAG * t,U32 pos)
 { swf_ResetReadBits(t);
   if (pos<=t->len) t->pos = pos;
@@ -146,7 +140,7 @@ int swf_GetBlock(TAG * t,U8 * b,int l)
   return l;
 }
 
-int swf_SetBlock(TAG * t,U8 * b,int l)
+int swf_SetBlock(TAG * t,const U8 * b,int l)
 // Appends Block to the end of Tagdata, returns size
 { U32 newlen = t->len + l;
   swf_ResetWriteBits(t);
@@ -210,17 +204,22 @@ U32 swf_GetBits(TAG * t,int nbits)
   if (!t->readBit) t->readBit = 0x80;
   while (nbits)
   { res<<=1;
+#ifdef DEBUG_RFXSWF
+    if (t->pos>=t->len) 
+    { fprintf(stderr,"GetBits() out of bounds: TagID = %i, pos=%d, len=%d\n",t->id, t->pos, t->len);
+      int i,m=t->len>10?10:t->len;
+      for(i=-1;i<m;i++) {
+        fprintf(stderr, "(%d)%02x ", i, t->data[i]);
+      } 
+      fprintf(stderr, "\n");
+      return res;
+    }
+#endif
     if (t->data[t->pos]&t->readBit) res|=1;
     t->readBit>>=1;
     nbits--;
     if (!t->readBit)
     { if (nbits) t->readBit = 0x80;
-      #ifdef DEBUG_RFXSWF
-      if (t->pos>=t->len) 
-      { fprintf(stderr,"GetBits() out of bounds: TagID = %i\n",t->id);
-        return res;
-      }
-      #endif
       t->pos++;
     }
   }
@@ -268,21 +267,205 @@ double swf_GetFixed(TAG * t)
 }
 void swf_SetFixed(TAG * t, double f)
 {
-  swf_SetU16(t, (U16)((f-(int)f)*65536));
-  swf_SetU16(t, (U16)f);
+  U16 fr = (U16)((f-(int)f)*65536);
+  swf_SetU16(t, fr);
+  swf_SetU16(t, (U16)f - (f<0 && fr!=0));
 }
 float swf_GetFixed8(TAG * t)
 {
   U8 low =  swf_GetU8(t);
   U8 high = swf_GetU8(t);
-  return high + low*(1/256.0);
+  return (float)(high + low*(1/256.0));
 }
 void swf_SetFixed8(TAG * t, float f)
 {
-  swf_SetU8(t, (U8)((f-(int)f)*256));
-  swf_SetU8(t, (U8)f);
+  U8 fr = (U8)((f-(int)f)*256);
+  swf_SetU8(t, fr);
+  swf_SetU8(t, (U8)f - (f<0 && fr!=0));
 }
 
+U32 swf_GetU30(TAG*tag)
+{
+    U32 shift = 0;
+    U32 s = 0;
+    int nr=0;
+    while(1) {
+	U8 b = swf_GetU8(tag);
+        nr++;
+	s|=(b&127)<<shift;
+	shift+=7;
+	if(!(b&128) || shift>=32)
+	    break;
+    }
+    /*int nr2= swf_SetU30(0, s);
+    if(nr!=nr2) {
+      printf("Unsigned value %d stored in %d bytes, I'd store it in %d bytes\n", s, nr, nr2);
+    }*/
+    return s;
+}
+
+int swf_SetU30(TAG*tag, U32 u)
+{
+    int nr = 0;
+    do {
+        if(tag)
+	  swf_SetU8(tag, (u&~0x7f?0x80:0) | (u&0x7F));
+	u>>=7;
+        nr++;
+    } while(u);
+    return nr;
+}
+
+void swf_SetABCU32(TAG*tag, U32 u)
+{
+  do {
+      swf_SetU8(tag, (u&~0x7f?0x80:0) | (u&0x7F));
+      u>>=7;
+  } while(u);
+}
+U32 swf_GetABCU32(TAG*tag)
+{
+  return swf_GetU30(tag);
+}
+void swf_SetABCS32(TAG*tag, S32 v)
+{
+  swf_SetABCU32(tag, v);
+}
+S32 swf_GetABCS32(TAG*tag)
+{
+  return swf_GetABCU32(tag);
+}
+
+#if 0
+
+/*The AVM2 spec is just plain wrong, claiming that S32 values are sign
+extended. They're not.
+This wastes up to 4 bytes for every negative value. */
+
+void swf_SetABCS32(TAG*tag, S32 s)
+{
+  printf("write S32: %d\n", s);
+    S32 neg = s<0?-1:0;
+    U8 sign = s<0?0x40:0;
+    while(1) {
+        U8 val = s&0x7f;
+        U8 vsign = s&0x40;
+	s>>=7;
+        neg>>=7;
+        if(s==neg && vsign==sign) {
+            /* if the value we now write has the same sign as s
+               and all the remaining bits are equal to the sign of s
+               too, stop writing */
+	    swf_SetU8(tag, val);
+            printf("put %02x\n", val);
+            break;
+        } else {
+            swf_SetU8(tag, 0x80 | val);
+            printf("put %02x\n", 0x80|val);
+        }
+    };
+}
+int swf_GetS30(TAG*tag)
+{
+    U32 shift = 0;
+    U32 s = 0;
+    int nr=0;
+    while(1) {
+	U8 b = swf_GetU8(tag);
+        nr++;
+	nt i,m=t->len>10?10:t->len;
+                for(i=0;i<m;i++) {
+                            fprintf(stderr, "%02x ", t->data[i]);
+                                    } 
+                        fprintf(stderr, "\n");
+                        s|=(b&127)<<shift;
+	shift+=7;
+	if(!(b&128) || shift>=32) {
+            if(b&64) {
+                if(shift<32) 
+                  s|=0xffffffff<<shift;
+            }
+	    break;
+        }
+    }
+    /* It's not uncommon for other applications (Flex for all negative numbers, and
+       Flash for -1) to generate a lot more bytes than would be necessary.
+       int nr2= swf_SetS30(0, s);
+    if(nr!=nr2) {
+      printf("Signed value %d stored in %d bytes, I'd store it in %d bytes\n", s, nr, nr2);
+    }*/
+    return s;
+}
+#endif
+
+int swf_SetU30String(TAG*tag, const char*str, int l)
+{
+    int len=0;
+    len+=swf_SetU30(tag, l);
+    len+=l;
+    swf_SetBlock(tag, (void*)str, l);
+    return len;
+}
+double swf_GetD64(TAG*tag)
+{
+    /* FIXME: this is not big-endian compatible */
+    double value = *(double*)&tag->data[tag->pos];
+    swf_GetU32(tag);
+    swf_GetU32(tag);
+    return value;
+}
+int swf_SetD64(TAG*tag, double v)
+{
+    /* FIXME: this is not big-endian compatible */
+    swf_SetU32(tag, ((U32*)&v)[0]);
+    swf_SetU32(tag, ((U32*)&v)[1]);
+    return 8;
+}
+int swf_GetU24(TAG*tag)
+{
+    int b1 = swf_GetU8(tag);
+    int b2 = swf_GetU8(tag);
+    int b3 = swf_GetU8(tag);
+    return b3<<16|b2<<8|b1;
+}
+int swf_GetS24(TAG*tag)
+{
+    int b1 = swf_GetU8(tag);
+    int b2 = swf_GetU8(tag);
+    int b3 = swf_GetU8(tag);
+    if(b3&0x80) {
+        return -1-((b3<<16|b2<<8|b1)^0xffffff);
+    } else {
+        return b3<<16|b2<<8|b1;
+    }
+}
+int swf_SetU24(TAG*tag, U32 v)
+{
+    if(tag) {
+        if(v&0xff000000)
+          fprintf(stderr, "Error: Overflow in swf_SetU24()\n");
+        swf_SetU8(tag, v);
+        swf_SetU8(tag, v>>8);
+        swf_SetU8(tag, v>>16);
+    }
+    return 3;
+}
+int swf_SetS24(TAG*tag, U32 v)
+{
+    if(tag) {
+        if(!(v&0xff000000))
+          return swf_SetU24(tag, v);
+        if((v&0xff000000)!=0xff000000) {
+          fprintf(stderr, "Error: Overflow in swf_SetS24()\n");
+        }
+        swf_SetU8(tag, v);
+        swf_SetU8(tag, v>>8);
+        swf_SetU8(tag, v>>16);
+    }
+    return 3;
+}
+
+
 int swf_SetRGB(TAG * t,RGBA * col)
 { if (!t) return -1;
   if (col)
@@ -326,7 +509,6 @@ void swf_GetRGBA(TAG * t, RGBA * col)
 
 void swf_GetGradient(TAG * tag, GRADIENT * gradient, char alpha)
 {
-    GRADIENT dummy;
     int t;
     if(!tag) {
       memset(gradient, 0, sizeof(GRADIENT));
@@ -335,8 +517,8 @@ void swf_GetGradient(TAG * tag, GRADIENT * gradient, char alpha)
     U8 num = swf_GetU8(tag) & 15;
     if(gradient) {
 	gradient->num = num;
-	gradient->rgba = rfx_calloc(sizeof(RGBA)*gradient->num);
-	gradient->ratios = rfx_calloc(sizeof(gradient->ratios[0])*gradient->num);
+	gradient->rgba = (RGBA*)rfx_calloc(sizeof(RGBA)*gradient->num);
+	gradient->ratios = (U8*)rfx_calloc(sizeof(gradient->ratios[0])*gradient->num);
     }
     for(t=0;t<num;t++)
     {
@@ -460,6 +642,20 @@ int swf_SetRect(TAG * t,SRECT * r)
   return 0;
 }
 
+SRECT swf_ClipRect(SRECT border, SRECT r)
+{
+    if(r.xmax > border.xmax) r.xmax = border.xmax;
+    if(r.ymax > border.ymax) r.ymax = border.ymax;
+    if(r.xmax < border.xmin) r.xmax = border.xmin;
+    if(r.ymax < border.ymin) r.ymax = border.ymin;
+    
+    if(r.xmin > border.xmax) r.xmin = border.xmax;
+    if(r.ymin > border.ymax) r.ymin = border.ymax;
+    if(r.xmin < border.xmin) r.xmin = border.xmin;
+    if(r.ymin < border.ymin) r.ymin = border.ymin;
+    return r;
+}
+
 void swf_ExpandRect(SRECT*src, SPOINT add)
 {
     if((src->xmin | src->ymin | src->xmax | src->ymax)==0) {
@@ -753,14 +949,23 @@ void  swf_SetPassword(TAG * t, const char * password)
     md5string = crypt_md5(password, salt);
 
     swf_SetU16(t,0);
-    swf_SetString(t, md5string);
+    swf_SetString(t, (U8*)md5string);
+} 
+
+void swf_SetString(TAG*t, const char* s) 
+{
+    if(!s) {
+        swf_SetU8(t, 0);
+    } else {
+        swf_SetBlock(t,s,strlen(s)+1);
+    }
 }
 
 int swf_VerifyPassword(TAG * t, const char * password)
 {
     char*md5string1, *md5string2;
     char*x;
-    char*md5, *salt;
+    char*salt;
     int n;
 
     if(t->len >= 5 && t->pos==0 && 
@@ -855,15 +1060,18 @@ TAG* swf_CopyTag(TAG*tag, TAG*to_copy)
     return tag;
 }
 
-int swf_DeleteTag(TAG * t)
-{ if (!t) return -1;
+TAG* swf_DeleteTag(SWF*swf, TAG * t)
+{
+  TAG*next = t->next;
 
+  if (swf && swf->firstTag==t) 
+    swf->firstTag = t->next;
   if (t->prev) t->prev->next = t->next;
   if (t->next) t->next->prev = t->prev;
 
   if (t->data) rfx_free(t->data);
   rfx_free(t);
-  return 0;
+  return next;
 }
 
 TAG * swf_ReadTag(reader_t*reader, TAG * prev)
@@ -930,7 +1138,12 @@ int swf_WriteTag2(writer_t*writer, TAG * t)
      t->id!=ST_DEFINEBITSJPEG&&t->id!=ST_DEFINEBITSJPEG2&&t->id!=ST_DEFINEBITSJPEG3);
 
   if (writer)
-  { if (short_tag)
+  {
+#ifdef MEASURE
+  int oldpos = writer->pos;
+#endif
+    
+    if (short_tag)
     { raw[0] = SWAP16(len|((t->id&0x3ff)<<6));
       if (writer->write(writer,raw,2)!=2)
       {
@@ -973,6 +1186,11 @@ int swf_WriteTag2(writer_t*writer, TAG * t)
     #ifdef DEBUG_RFXSWF
       else if (t->len) fprintf(stderr,"WriteTag(): Tag Data Error, id=%i\n",t->id);
     #endif
+
+#ifdef MEASURE
+  writer->flush(writer);
+  printf("TAG %s costs %d bytes\n", swf_TagGetName(t), writer->pos-oldpos);
+#endif
   }
 
   return t->len+(short_tag?2:6);
@@ -1060,7 +1278,7 @@ void swf_UnFoldSprite(TAG * t)
 void swf_FoldSprite(TAG * t)
 {
   TAG*sprtag=t,*tmp;
-  U16 id,frames,tmpid;
+  U16 id,frames;
   int level;
   if(t->id!=ST_DEFINESPRITE)
       return;
@@ -1121,7 +1339,7 @@ void swf_FoldSprite(TAG * t)
     if(t->id == ST_END)
 	level--;
     t = swf_NextTag(t);
-    swf_DeleteTag(tmp);
+    swf_DeleteTag(0, tmp);
   } 
   while (t && level);
   if(level)
@@ -1243,6 +1461,7 @@ int swf_ReadSWF2(reader_t*reader, SWF * swf)   // Reads SWF to memory (malloc'ed
 	reader_init_zlibinflate(&zreader, reader);
 	reader = &zreader;
     }
+    swf->compressed = 0; // derive from version number from now on
 
     reader_GetRect(reader, &swf->movieSize);
     reader->read(reader, &swf->frameRate, 2);
@@ -1252,7 +1471,13 @@ int swf_ReadSWF2(reader_t*reader, SWF * swf)   // Reads SWF to memory (malloc'ed
 
     /* read tags and connect to list */
     t = &t1;
-    while (t) t = swf_ReadTag(reader,t);
+    while (t) {
+      t = swf_ReadTag(reader,t);
+      if(t && t->id == ST_FILEATTRIBUTES) {
+        swf->fileAttributes = swf_GetU32(t);
+        swf_ResetReadBits(t);
+      }
+    }
     swf->firstTag = t1.next;
     t1.next->prev = NULL;
   }
@@ -1260,6 +1485,19 @@ int swf_ReadSWF2(reader_t*reader, SWF * swf)   // Reads SWF to memory (malloc'ed
   return reader->pos;
 }
 
+SWF* swf_OpenSWF(char*filename)
+{
+  int fi = open(filename, O_RDONLY|O_BINARY);
+  if(fi<0) {
+      fprintf(stderr, "Failed to open %s\n", filename);
+      return 0;
+  }
+  SWF* swf = rfx_alloc(sizeof(SWF));
+  swf_ReadSWF(fi, swf);
+  close(fi);
+  return swf;
+}
+
 int swf_ReadSWF(int handle, SWF * swf)
 {
   reader_t reader;
@@ -1267,6 +1505,80 @@ int swf_ReadSWF(int handle, SWF * swf)
   return swf_ReadSWF2(&reader, swf);
 }
 
+void swf_ReadABCfile(char*filename, SWF*swf)
+{
+    memset(swf, 0, sizeof(SWF));
+    swf->fileVersion=9;
+    swf->fileAttributes=FILEATTRIBUTE_AS3; //as3
+    TAG*tag = swf->firstTag = swf_InsertTag(0, ST_RAWABC);
+    memfile_t*file = memfile_open(filename);
+    swf_SetBlock(tag, file->data, file->len);
+    memfile_close(file);
+}
+
+int no_extra_tags = 0;
+
+int WriteExtraTags(SWF*swf, writer_t*writer)
+{
+    TAG*t = swf->firstTag;
+    TAG* has_fileattributes=0;
+    int has_scenedescription=0;
+    int has_version_8_action=0;
+    int has_version_9_action=0;
+    int len = 0;
+    while(t) {
+        if(t->id == ST_FILEATTRIBUTES)
+            has_fileattributes = t;
+        if(t->id == ST_SCENEDESCRIPTION)
+            has_scenedescription = 1;
+        if(t->id == ST_DOABC) 
+            has_version_9_action=1;
+        /* FIXME: this doesn't yet find actionscript in buttons */
+        if(t->id == ST_DOACTION || t->id == ST_DOINITACTION) 
+            has_version_8_action=1;
+        if(t->id == ST_PLACEOBJECT2 && t->len && (t->data[0]&0x80))
+            has_version_8_action=1;
+        t = t->next;
+    }
+    if(has_version_8_action && has_version_9_action) {
+        fprintf(stderr, "Warning: File contains both flash 8 and flash 9 actionscript\n");
+    }
+
+    if(swf->fileVersion >= 9) {
+        if(!has_fileattributes) {
+            U32 flags = swf->fileAttributes|FILEATTRIBUTE_AS3; // 16 = has symbolclass tag | 8 = actionscript3 | 1 = usenetwork
+            if(has_version_8_action && !has_version_9_action)
+                flags &= ~FILEATTRIBUTE_AS3;
+            TAG*fileattrib = swf_InsertTag(0, ST_FILEATTRIBUTES);
+            swf_SetU32(fileattrib, flags);
+            if(writer) {
+                if(swf_WriteTag2(writer, fileattrib)<0) 
+                    return -1;
+            } else {
+                len += swf_WriteTag(-1,fileattrib);
+            }
+            swf_DeleteTag(0, fileattrib);
+        } else {
+            if(swf_WriteTag2(writer, has_fileattributes)<0) 
+                return -1;
+        }
+        if(0 && !has_scenedescription) {
+            TAG*scene = swf_InsertTag(0, ST_SCENEDESCRIPTION);
+            swf_SetU16(scene, 1);
+            swf_SetString(scene, (U8*)"Scene 1");
+            swf_SetU8(scene, 0);
+            if(writer) {
+                if(swf_WriteTag2(writer, scene)<0) 
+                    return -1;
+            } else {
+                len += swf_WriteTag(-1,scene);
+            }
+            swf_DeleteTag(0, scene);
+        }
+    }
+    return len;
+}
+
 int  swf_WriteSWF2(writer_t*writer, SWF * swf)     // Writes SWF to file, returns length or <0 if fails
 { U32 len;
   TAG * t;
@@ -1274,34 +1586,14 @@ int  swf_WriteSWF2(writer_t*writer, SWF * swf)     // Writes SWF to file, return
   writer_t zwriter;
   int fileSize = 0;
   int inSprite = 0;
-  int writer_lastpos = 0;
   int ret;
+  writer_t*original_writer = writer;
+  int writer_lastpos = 0;
     
   if (!swf) return -1;
   if (!writer) return -1; // the caller should provide a nullwriter, not 0, for querying SWF size
 
-  if(writer) writer_lastpos = writer->pos;
-
-  if(swf->fileVersion >= 8) {
-      if ((swf->firstTag && swf->firstTag->id != ST_FILEATTRIBUTES) &&
-	  (!swf->firstTag->next || swf->firstTag->next->id != ST_FILEATTRIBUTES)) 
-      {
-	  U32 flags = 0; // | 128 = usenetwork, | 8 = hasmetadata
-	  swf_SetU32(swf_InsertTagBefore(swf, swf->firstTag,ST_FILEATTRIBUTES),flags);
-      }
-  }
-
-  // Insert REFLEX Tag
-
-#ifdef INSERT_RFX_TAG
-
-  if ((swf->firstTag && swf->firstTag->id != ST_REFLEX) &&
-      (!swf->firstTag->next || swf->firstTag->next->id != ST_REFLEX)) 
-  {
-      swf_SetBlock(swf_InsertTagBefore(swf, swf->firstTag,ST_REFLEX),"rfx",3);
-  }
-
-#endif // INSERT_RFX_TAG
+  if(original_writer) writer_lastpos = original_writer->pos;
 
   // Count Frames + File Size
 
@@ -1309,6 +1601,9 @@ int  swf_WriteSWF2(writer_t*writer, SWF * swf)     // Writes SWF to file, return
   t = swf->firstTag;
   frameCount = 0;
 
+  if(swf->firstTag && !no_extra_tags) {
+    len += WriteExtraTags(swf, 0);
+  }
   while(t) {
       len += swf_WriteTag(-1,t);
       if(t->id == ST_DEFINESPRITE && !swf_IsFolded(t)) inSprite++;
@@ -1357,11 +1652,10 @@ int  swf_WriteSWF2(writer_t*writer, SWF * swf)     // Writes SWF to file, return
        It also means that we don't initialize our own zlib
        writer, but assume the caller provided one.
      */
-      if(swf->compressed) {
+      if(swf->compressed==1 || (swf->compressed==0 && swf->fileVersion>=6)) {
 	char*id = "CWS";
 	writer->write(writer, id, 3);
-      }
-      else {
+      } else {
 	char*id = "FWS";
 	writer->write(writer, id, 3);
       }
@@ -1370,7 +1664,7 @@ int  swf_WriteSWF2(writer_t*writer, SWF * swf)     // Writes SWF to file, return
       PUT32(b4, swf->fileSize);
       writer->write(writer, b4, 4);
       
-      if(swf->compressed) {
+      if(swf->compressed==1 || (swf->compressed==0 && swf->fileVersion>=6)) {
 	writer_init_zlibdeflate(&zwriter, writer);
 	writer = &zwriter;
       }
@@ -1391,15 +1685,22 @@ int  swf_WriteSWF2(writer_t*writer, SWF * swf)     // Writes SWF to file, return
       return -1;
     }
 
-    t = swf->firstTag;
-    while (t)
-    { if (swf_WriteTag2(writer, t)<0) return -1;
-      t = swf_NextTag(t);
+    if(swf->firstTag && !no_extra_tags) {
+        WriteExtraTags(swf, writer);
     }
-    if(swf->compressed) {
+    t = swf->firstTag;
+
+    while (t) { 
+        if(no_extra_tags || t->id != ST_FILEATTRIBUTES) {
+          if(swf_WriteTag2(writer, t)<0) 
+            return -1;
+        }
+        t = t->next;
+    }
+    if(swf->compressed==1 || (swf->compressed==0 && swf->fileVersion>=6) || swf->compressed==8) {
       if(swf->compressed != 8) {
 	zwriter.finish(&zwriter);
-	return writer->pos - writer_lastpos;
+	return original_writer->pos - writer_lastpos;
       }
       return (int)fileSize;
     } else {
@@ -1412,7 +1713,6 @@ int  swf_WriteSWF(int handle, SWF * swf)     // Writes SWF to file, returns leng
 {
   writer_t writer;
   int len = 0;
-  swf->compressed = 0;
   
   if(handle<0) {
     writer_init_nullwriter(&writer);
@@ -1425,22 +1725,6 @@ int  swf_WriteSWF(int handle, SWF * swf)     // Writes SWF to file, returns leng
   return len;
 }
 
-int  swf_WriteSWC(int handle, SWF * swf)     // Writes SWF to file, returns length or <0 if fails
-{
-  writer_t writer;
-  int len = 0;
-  swf->compressed = 1;
-
-  if(handle<0) {
-    writer_init_nullwriter(&writer);
-    len = swf_WriteSWF2(&writer, swf);
-  }
-  writer_init_filewriter(&writer, handle);
-  len = swf_WriteSWF2(&writer, swf);
-  writer.finish(&writer);
-  return len;
-}
-
 int swf_WriteHeader2(writer_t*writer,SWF * swf)
 {
   SWF myswf;
@@ -1477,7 +1761,7 @@ int swf_WriteCGI(SWF * swf)
 
 SWF* swf_CopySWF(SWF*swf)
 {
-    SWF*nswf = rfx_alloc(sizeof(SWF));
+    SWF*nswf = (SWF*)rfx_alloc(sizeof(SWF));
     TAG*tag, *ntag;
     memcpy(nswf, swf, sizeof(SWF));
     nswf->firstTag = 0;
@@ -1506,17 +1790,18 @@ void swf_FreeTags(SWF * swf)                 // Frees all malloc'ed memory for t
 
 // include advanced functions
 
-#include "modules/swfdump.c"
-#include "modules/swfshape.c"
-#include "modules/swftext.c"
-#include "modules/swffont.c"
-#include "modules/swfobject.c"
-#include "modules/swfbutton.c"
-#include "modules/swftools.c"
-#include "modules/swfcgi.c"
-#include "modules/swfbits.c"
-#include "modules/swfaction.c"
-#include "modules/swfsound.c"
-#include "modules/swfdraw.c"
-#include "modules/swfrender.c"
-#include "modules/swffilter.c"
+//#include "modules/swfdump.c"
+//#include "modules/swfshape.c"
+//#include "modules/swftext.c"
+//#include "modules/swffont.c"
+//#include "modules/swfobject.c"
+//#include "modules/swfbutton.c"
+//#include "modules/swftools.c"
+//#include "modules/swfcgi.c"
+//#include "modules/swfbits.c"
+//#include "modules/swfaction.c"
+//#include "modules/swfabc.c"
+//#include "modules/swfsound.c"
+//#include "modules/swfdraw.c"
+//#include "modules/swfrender.c"
+//#include "modules/swffilter.c"
