@@ -5,7 +5,7 @@
    Part of the swftools package.
 
    Copyright (c) 2000, 2001 Rainer Bhme <rfxswf@reflex-studio.de>
- 
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -19,20 +19,18 @@
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
- 
+
 
 #ifndef __RFX_SWF_INCLUDED__
 #define __RFX_SWF_INCLUDED__
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
 #include <fcntl.h>
 #include <ctype.h>
 #include "../config.h"
@@ -41,16 +39,20 @@ extern "C" {
 #include "./mem.h"
 #include "./types.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #define DEBUG_RFXSWF
 #ifdef RFXSWF_DISABLESOUND
 #define NO_MP3
 #endif
 
-typedef         signed long     SFIXED;
-typedef         signed long     SCOORD;
+typedef signed SFIXED;
+typedef signed SCOORD;
 
-#define SCOORD_MAX 0x7fffffff
-#define SCOORD_MIN -0x80000000
+#define SCOORD_MAX 0x7fffffffl
+#define SCOORD_MIN -0x80000000l
 
 // Basic Structures
 
@@ -123,13 +125,13 @@ typedef struct _TAG             // NEVER access a Tag-Struct directly !
 
   U8            readBit;        // for Bit-Manipulating Functions [read]
   U8            writeBit;       // [write]
-  
+
 } TAG;
 
 #define swf_ResetReadBits(tag)   if (tag->readBit)  { tag->pos++; tag->readBit = 0; }
 #define swf_ResetWriteBits(tag)  if (tag->writeBit) { tag->writeBit = 0; }
 
-typedef struct _SOUNDINFO 
+typedef struct _SOUNDINFO
 {
     U8 stop;
     U8 nomultiple; //continue playing if already started
@@ -146,6 +148,9 @@ typedef struct _SOUNDINFO
     U32* right;
 } SOUNDINFO;
 
+#define FILEATTRIBUTE_USENETWORK 1
+#define FILEATTRIBUTE_AS3 8
+#define FILEATTRIBUTE_SYMBOLCLASS 16
 typedef struct _SWF
 { U8            fileVersion;
   U8		compressed;     // SWF or SWC?
@@ -154,18 +159,20 @@ typedef struct _SWF
   U16           frameRate;
   U16           frameCount;     // valid after load and save
   TAG *         firstTag;
+  U32           fileAttributes; // for SWFs >= Flash9
 } SWF;
 
 // Basic Functions
 
+SWF* swf_OpenSWF(char*filename);
 int  swf_ReadSWF2(reader_t*reader, SWF * swf);   // Reads SWF via callback
 int  swf_ReadSWF(int handle,SWF * swf);     // Reads SWF to memory (malloc'ed), returns length or <0 if fails
 int  swf_WriteSWF2(writer_t*writer, SWF * swf);     // Writes SWF via callback, returns length or <0 if fails
 int  swf_WriteSWF(int handle,SWF * swf);    // Writes SWF to file, returns length or <0 if fails
-int  swf_WriteSWC(int handle, SWF * swf);   // for convenience, equal to swf->compressed=1;swf_WriteSWF(..)
 int  swf_WriteCGI(SWF * swf);               // Outputs SWF with valid CGI header to stdout
 void swf_FreeTags(SWF * swf);               // Frees all malloc'ed memory for swf
 SWF* swf_CopySWF(SWF*swf);
+void swf_ReadABCfile(char*filename, SWF*swf);
 
 // for streaming:
 int  swf_WriteHeader(int handle,SWF * swf);    // Writes Header of swf to file
@@ -188,15 +195,15 @@ int swf_IsFolded(TAG*tag);
 void swf_OptimizeTagOrder(SWF*swf);
 
 // basic routines:
-    
+
 TAG * swf_InsertTag(TAG * after,U16 id);    // updates frames, if necessary
 TAG * swf_InsertTagBefore(SWF*swf, TAG * before,U16 id);     // like InsertTag, but insert tag before argument
-int   swf_DeleteTag(TAG * t);
+TAG * swf_DeleteTag(SWF*swf, TAG * t);
 
 void  swf_ClearTag(TAG * t);                //frees tag data
 void  swf_ResetTag(TAG*tag, U16 id);        //set's tag position and length to 0, without freeing it
 TAG*  swf_CopyTag(TAG*tag, TAG*to_copy);     //stores a copy of another tag into this taglist
-    
+
 void  swf_SetTagPos(TAG * t,U32 pos);       // resets Bitcount
 U32   swf_GetTagPos(TAG * t);
 
@@ -210,9 +217,11 @@ U8*   swf_GetTagLenPtr(TAG * t);
 U32   swf_GetBits(TAG * t,int nbits);
 S32   swf_GetSBits(TAG * t,int nbits);
 int   swf_SetBits(TAG * t,U32 v,int nbits);
+int   swf_CountUBits(U32 v,int nbits);
+int   swf_CountBits(U32 v,int nbits);
 
 int   swf_GetBlock(TAG * t,U8 * b,int l);   // resets Bitcount
-int   swf_SetBlock(TAG * t,U8 * b,int l);
+int   swf_SetBlock(TAG * t,const U8 * b,int l);
 
 U8    swf_GetU8(TAG * t);                   // resets Bitcount
 U16   swf_GetU16(TAG * t);
@@ -221,13 +230,29 @@ U32   swf_GetU32(TAG * t);
 void  swf_GetRGB(TAG * t, RGBA * col);
 void  swf_GetRGBA(TAG * t, RGBA * col);
 void  swf_GetGradient(TAG * t, GRADIENT * gradient, char alpha);
+void  swf_SetGradient(TAG * tag, GRADIENT * gradient, char alpha);
 void  swf_FreeGradient(GRADIENT* gradient);
 char* swf_GetString(TAG*t);
 int   swf_SetU8(TAG * t,U8 v);              // resets Bitcount
 int   swf_SetU16(TAG * t,U16 v);
 void  swf_SetS16(TAG * t,int v);
 int   swf_SetU32(TAG * t,U32 v);
-#define swf_SetString(t,s)  swf_SetBlock(t,s,strlen(s)+1)
+void  swf_SetString(TAG*t,const char*s);
+
+/* abc datatypes */
+U32 swf_GetU30(TAG*tag);
+int swf_SetU30(TAG*tag, U32 u);
+void swf_SetABCU32(TAG*tag, U32 u);
+U32 swf_GetABCU32(TAG*tag);
+void swf_SetABCS32(TAG*tag, S32 v);
+S32 swf_GetABCS32(TAG*tag);
+int    swf_SetU30String(TAG*tag, const char*str, int len);
+double swf_GetD64(TAG*tag);
+int    swf_SetD64(TAG*tag, double v);
+int    swf_GetU24(TAG*tag);
+int    swf_GetS24(TAG*tag);
+int    swf_SetU24(TAG*tag, U32 v);
+int    swf_SetS24(TAG*tag, U32 v);
 
 //int   swf_GetPoint(TAG * t,SPOINT * p);     // resets Bitcount
 int   swf_GetRect(TAG * t,SRECT * r);
@@ -251,6 +276,7 @@ int   swf_VerifyPassword(TAG * t, const char * password);
 
 // helper functions:
 
+SRECT swf_ClipRect(SRECT border, SRECT r);
 void swf_ExpandRect(SRECT*src, SPOINT add);
 void swf_ExpandRect2(SRECT*src, SRECT*add);
 void swf_ExpandRect3(SRECT*src, SPOINT center, int radius);
@@ -270,7 +296,7 @@ SRECT swf_TurnRect(SRECT r, MATRIX* m);
 #define ST_PLACEOBJECT          4
 #define ST_REMOVEOBJECT         5
 #define ST_DEFINEBITS           6
-#define ST_DEFINEBITSJPEG       6 
+#define ST_DEFINEBITSJPEG       6
 #define ST_DEFINEBUTTON         7
 #define ST_JPEGTABLES           8
 #define ST_SETBACKGROUNDCOLOR   9
@@ -323,13 +349,19 @@ SRECT swf_TurnRect(SRECT r, MATRIX* m);
 #define ST_FILEATTRIBUTES	69 /* version 8 (required)- */
 #define ST_PLACEOBJECT3		70 /* version 8 */
 #define ST_IMPORTASSETS2	71 /* version 8 */
+#define ST_RAWABC               72 /* version 9, used by flex */
 #define ST_DEFINEFONTALIGNZONES 73 /* version 8 */
 #define ST_CSMTEXTSETTINGS	74 /* version 8 */
 #define ST_DEFINEFONT3		75 /* version 8 */
+#define ST_SYMBOLCLASS		76 /* version 9 */
 #define ST_METADATA		77 /* version 8 */
 #define ST_DEFINESCALINGGRID    78 /* version 8 */
+#define ST_DOABC		82 /* version 9 */
 #define ST_DEFINESHAPE4		83 /* version 8 */
 #define ST_DEFINEMORPHSHAPE2    84 /* version 8 */
+#define ST_SCENEDESCRIPTION	86 /* version 9 */
+#define ST_DEFINEBINARY		87 /* version 9 */
+#define ST_DEFINEFONTNAME	88 /* version 9 */
 
 /* custom tags- only valid for swftools */
 #define ST_REFLEX              777 /* to identify generator software */
@@ -353,23 +385,23 @@ typedef struct _LINESTYLE
 typedef struct _FILLSTYLE
 { U8	    type;
   RGBA	    color;
-  MATRIX    m; 
+  MATRIX    m;
   U16	    id_bitmap;
   GRADIENT  gradient;
 } FILLSTYLE;
-     
+
 typedef struct _SHAPE           // NEVER access a Shape-Struct directly !
-{                 
+{
   struct
   { LINESTYLE * data;
     U16         n;
   } linestyle;
-                  
-  struct                    
+
+  struct
   { FILLSTYLE * data;
     U16         n;
   } fillstyle;
- 
+
   struct
   { U16         fill;
     U16         line;
@@ -392,9 +424,10 @@ typedef struct _SHAPE2
     SRECT* bbox; // may be NULL
 } SHAPE2;
 
+enum SHAPELINETYPE {moveTo, lineTo, splineTo};
 typedef struct _SHAPELINE
 {
-    enum {moveTo, lineTo, splineTo} type;
+    enum SHAPELINETYPE type;
     SCOORD x,y;
     SCOORD sx,sy; //only if type==splineTo
     int fillstyle0;
@@ -431,6 +464,10 @@ int   swf_ShapeSetLine(TAG * t,SHAPE * s,S32 x,S32 y);
 int   swf_ShapeSetCurve(TAG * t,SHAPE * s,S32 x,S32 y,S32 ax,S32 ay);
 int   swf_ShapeSetCircle(TAG * t,SHAPE * s,S32 x,S32 y,S32 rx,S32 ry);
 int   swf_ShapeSetEnd(TAG * t);
+int   swf_SetShapeStyleCount(TAG * t,U16 n);
+int   swf_SetFillStyle(TAG * t,FILLSTYLE * f);
+int   swf_SetLineStyle(TAG * t,LINESTYLE * l);
+
 
 void  swf_ShapeSetBitmapRect(TAG * t, U16 gfxid, int width, int height);
 
@@ -446,7 +483,7 @@ void	swf_DumpShape(SHAPE2*shape2);
 void swf_ParseDefineShape(TAG*tag, SHAPE2*shape);
 void swf_SetShape2(TAG*tag, SHAPE2*shape2);
 
-void swf_RecodeShapeData(U8*data, int bitlen, int in_bits_fill, int in_bits_line, 
+void swf_RecodeShapeData(U8*data, int bitlen, int in_bits_fill, int in_bits_line,
                          U8**destdata, U32*destbitlen, int out_bits_fill, int out_bits_line);
 
 // swfdraw.c
@@ -486,12 +523,13 @@ typedef struct _FONTUSAGE
 { int* chars;
   char is_reduced;
   int used_glyphs;
+  int glyphs_specified;
 } FONTUSAGE;
 
 #define FONT_STYLE_BOLD 1
 #define FONT_STYLE_ITALIC 2
 #define FONT_ENCODING_UNICODE 1
-#define FONT_ENCODING_ANSI 2 
+#define FONT_ENCODING_ANSI 2
 #define FONT_ENCODING_SHIFTJIS 4
 
 typedef struct _SWFFONT
@@ -499,9 +537,9 @@ typedef struct _SWFFONT
   U8		version; // 0 = not set, 1 = definefont, 2 = definefont2
   U8 *          name;
   SWFLAYOUT *   layout;
-  U16		numchars;
-  U16		maxascii; // highest mapped ascii value
-  
+  int           numchars;
+  int           maxascii; // highest mapped ascii/unicode value
+
   U8		style;
   U8		encoding;
 
@@ -553,15 +591,22 @@ int swf_FontEnumerate(SWF * swf,void (*FontCallback) (void*,U16,U8*), void*self)
 int swf_FontExtract(SWF * swf,int id,SWFFONT ** f);
 // Fetches all available information from DefineFont, DefineFontInfo, DefineText, ...
 // id = FontID, id=0 -> Extract first Font
+int swf_FontExtract_DefineFont2(int id, SWFFONT * font, TAG * tag);
+int swf_FontExtract_DefineFontInfo(int id, SWFFONT * f, TAG * t);
+int swf_FontExtract_DefineFont(int id, SWFFONT * f, TAG * t);
+int swf_FontExtract_GlyphNames(int id, SWFFONT * f, TAG * tag);
 
 int swf_FontIsItalic(SWFFONT * f);
 int swf_FontIsBold(SWFFONT * f);
 
 int swf_FontSetID(SWFFONT * f,U16 id);
 int swf_FontReduce(SWFFONT * f);
+int swf_FontReduce_swfc(SWFFONT * f);
 
 int swf_FontInitUsage(SWFFONT * f);
 int swf_FontUseGlyph(SWFFONT * f, int glyph);
+int swf_FontUseAll(SWFFONT* f);
+int swf_FontUseUTF8(SWFFONT * f, U8 * s);
 int swf_FontUse(SWFFONT* f,U8 * s);
 
 int swf_FontSetDefine(TAG * t,SWFFONT * f);
@@ -574,7 +619,7 @@ void swf_FontAddLayout(SWFFONT * f, int ascent, int descent, int leading);
 int swf_ParseDefineText(TAG * t, void(*callback)(void*self, int*chars, int*xpos, int nr, int fontid, int fontsize, int xstart, int ystart, RGBA* color), void*self);
 
 void swf_WriteFont(SWFFONT* font, char* filename);
-SWFFONT* swf_ReadFont(char* filename);
+SWFFONT* swf_ReadFont(const char* filename);
 
 void swf_FontFree(SWFFONT * f);
 
@@ -591,18 +636,18 @@ int swf_TextPrintDefineText(TAG * t,SWFFONT * f);
 void swf_FontPrepareForEditText(SWFFONT * f);
 
 /* notice: if you set the fontid, make sure you call swf_FontPrepareForEditText() for the font first */
-void swf_SetEditText(TAG*tag, U16 flags, SRECT r, char*text, RGBA*color, 
-	int maxlength, U16 font, U16 height, EditTextLayout*layout, char*variable);
+void swf_SetEditText(TAG*tag, U16 flags, SRECT r, const char*text, RGBA*color,
+	int maxlength, U16 font, U16 height, EditTextLayout*layout, const char*variable);
 
-SRECT swf_SetDefineText(TAG*tag, SWFFONT*font, RGBA*rgb, char*text, int scale);
+SRECT swf_SetDefineText(TAG*tag, SWFFONT*font, RGBA*rgb, const char*text, int scale);
 
-void swf_DrawText(drawer_t*draw, SWFFONT*font, int size, char*text);
+void swf_DrawText(drawer_t*draw, SWFFONT*font, int size, const char*text);
 
 // swffont.c
 
-SWFFONT* swf_LoadTrueTypeFont(char*filename);
-SWFFONT* swf_LoadT1Font(char*filename);
-SWFFONT* swf_LoadFont(char*filename);
+SWFFONT* swf_LoadTrueTypeFont(const char*filename);
+SWFFONT* swf_LoadT1Font(const char*filename);
+SWFFONT* swf_LoadFont(const char*filename);
 
 void swf_SetLoadFontParameters(int scale, int skip_unused, int full_unicode);
 
@@ -610,86 +655,12 @@ void swf_SetLoadFontParameters(int scale, int skip_unused, int full_unicode);
 
 void swf_DumpHeader(FILE * f,SWF * swf);
 void swf_DumpMatrix(FILE * f,MATRIX * m);
-void swf_DumpTag(FILE * f,TAG * t); 
+void swf_DumpTag(FILE * f,TAG * t);
 void swf_DumpSWF(FILE * f,SWF*swf);
 void swf_DumpGradient(FILE* f, GRADIENT*gradient);
 char* swf_TagGetName(TAG*tag);
 void swf_DumpFont(SWFFONT * font);
 
-// swfbutton.c
-
-// Button States
-
-#define BS_HIT          0x08
-#define BS_DOWN         0x04
-#define BS_OVER         0x02
-#define BS_UP           0x01
-
-// Button Conditions
-
-/* missing: IDLE_OUTDOWN 
-            OUTDOWN_OVERUP
-	    OVERUP_OUTDOWN
-*/
-#define BC_OVERDOWN_IDLE        0x0100
-#define BC_IDLE_OVERDOWN        0x0080
-#define BC_OUTDOWN_IDLE         0x0040
-#define BC_OUTDOWN_OVERDOWN     0x0020
-#define BC_OVERDOWN_OUTDOWN     0x0010
-#define BC_OVERDOWN_OVERUP      0x0008
-#define BC_OVERUP_OVERDOWN      0x0004
-#define BC_OVERUP_IDLE          0x0002
-#define BC_IDLE_OVERUP          0x0001
-
-#define BC_KEY(c) (c<<9)
-
-#define BC_CURSORLEFT           0x0200
-#define BC_CURSORRIGHT          0x0400
-#define BC_POS1                 0x0600
-#define BC_END                  0x0800
-#define BC_INSERT               0x0a00
-#define BC_DELETE               0x0c00
-#define BC_CLEAR		0x0e00
-#define BC_BACKSPACE            0x1000
-#define BC_ENTER                0x1a00
-#define BC_CURSORUP             0x1c00
-#define BC_CURSORDOWN           0x1e00
-#define BC_PAGEUP               0x2000
-#define BC_PAGEDOWN             0x2200
-#define BC_TAB                  0x2400
-#define BC_ESCAPE		0x3600
-#define BC_SPACE                0x4000
-
-/* these are probably only valid with linux:
-   Ctrl-A	 0x0200
-   Ctrl-X	 0x3000
-   Ctrl-Y	 0x3200
-   Ctrl-Z	 0x3400
-   Escape/Ctrl-[ 0x3600
-   Ctrl-\	 0x3800
-   Ctrl-]	 0x3a00
-   Ctrl-^	 0x3c00
-   Ctrl-/	 0x3e00
-   */
-
-/* everything above 0x4000 is standard ascii:
-   0x4000 ' ' 0x4200 '!' 0x4600 '#' 0x4800 '$' 0x4a00 '%' 0x4c00 '&' ...
-   0x6000 '0' ... 0x7200 '9' 
-   0x8000 '@' 
-   0x8200 'A' ...  0xb400 'Z' 
-   ...
-   0xfc00 '~'
- */
-
-// Button Flag
-
-#define BF_TRACKMENU            0x01
-
-int swf_ButtonSetRecord(TAG * t,U8 state,U16 id,U16 layer,MATRIX * m,CXFORM * cx);
-int swf_ButtonSetCondition(TAG * t,U16 condition); // for DefineButton2
-int swf_ButtonSetFlags(TAG * t,U8 flags);  // necessary for DefineButton2
-int swf_ButtonPostProcess(TAG * t,int anz_action); // Set all offsets in DefineButton2-Tags (how many conditions to process)
-
 // swfbits.c
 
 int swf_ImageHasAlpha(RGBA*img, int width, int height);
@@ -701,9 +672,9 @@ int swf_SetJPEGBitsLines(JPEGBITS * jpegbits,U8 ** data,int n); // deprecated
 int swf_SetJPEGBitsLine(JPEGBITS * jpegbits,U8 * data); // deprecated
 int swf_SetJPEGBitsFinish(JPEGBITS * jpegbits); // deprecated
 
-void swf_GetJPEGSize(char * fname, int*width, int*height);
+void swf_GetJPEGSize(const char * fname, int*width, int*height);
 
-int swf_SetJPEGBits(TAG * t,char * fname,int quality);
+int swf_SetJPEGBits(TAG * t,const char * fname,int quality);
 void swf_SetJPEGBits2(TAG * t,U16 width,U16 height,RGBA * bitmap,int quality);
 int swf_SetJPEGBits3(TAG * tag,U16 width,U16 height,RGBA* bitmap, int quality);
 RGBA* swf_JPEG2TagToImage(TAG*tag, int*width, int*height);
@@ -763,7 +734,7 @@ MATRIX * swf_MatrixMapTriangle(MATRIX * m,int dx,int dy,
                     int x0,int y0,int x1,int y1,int x2,int y2);
 int swf_GetNumUsedIDs(TAG * t);
 void swf_GetUsedIDs(TAG * t, int * positions);
-void swf_Relocate(SWF*swf, char*bitmap); // bitmap is 65536 bytes, bitmap[a]==0 means id a is free
+char swf_Relocate(SWF*swf, char*bitmap); // bitmap is 65536 bytes, bitmap[a]==0 means id a is free
 void swf_RelocateDepth(SWF*swf, char*bitmap); // bitmap is 65536 bytes, bitmap[d]==0 means depth d is free
 
 TAG* swf_Concatenate (TAG*list1,TAG*list2); // warning: both list1 and list2 are invalid after this call.
@@ -774,9 +745,19 @@ RGBA swf_GetSWFBackgroundColor(SWF*swf);
 
 void swf_uncgi();  // same behaviour as Steven Grimm's uncgi-library
 
+// as3/abc.c
+void*swf_ReadABC(TAG*tag);
+void*swf_DumpABC(FILE*fo, void*code, char*prefix);
+void swf_WriteABC(TAG*tag, void*code);
+void swf_FreeABC(void*code);
+void swf_AddButtonLinks(SWF*swf, char stop_each_frame, char events);
+TAG*swf_AddAS3FontDefine(TAG*tag, U16 id, char*fontname);
+#include "as3/compiler.h"
+#include "as3/import.h"
+
 // swfaction.c
 
-typedef struct _ActionTAG 
+typedef struct _ActionTAG
 { U8            op;
   U16           len;
   U8 *          data;
@@ -804,6 +785,8 @@ void swf_ActionEnumerateStrings(ActionTAG*atag, char*(*callback)(char*));
 // using action/actioncompiler.h:
 ActionTAG* swf_ActionCompile(const char* source, int version);
 
+#define ACTION__GOTOFRAME      0x81
+#define ACTION__GETURL         0x83
 ActionTAG* action_End(ActionTAG*atag);
 ActionTAG* action_NextFrame(ActionTAG*atag);
 ActionTAG* action_PreviousFrame(ActionTAG*atag);
@@ -878,15 +861,15 @@ ActionTAG* action_BitLShift(ActionTAG*atag);
 ActionTAG* action_BitRShift(ActionTAG*atag);
 ActionTAG* action_BitURShift(ActionTAG*atag);
 ActionTAG* action_GotoFrame(ActionTAG*atag, U16 frame);
-ActionTAG* action_GetUrl(ActionTAG*atag, char* url, char* label);
+ActionTAG* action_GetUrl(ActionTAG*atag, const char* url, char* label);
 ActionTAG* action_StoreRegister(ActionTAG*atag, U8 reg);
 ActionTAG* action_Constantpool(ActionTAG*atag, char* constantpool);
 ActionTAG* action_WaitForFrame(ActionTAG*atag, U16 frame, U8 skip);
-ActionTAG* action_SetTarget(ActionTAG*atag, char* target);
+ActionTAG* action_SetTarget(ActionTAG*atag, const char* target);
 ActionTAG* action_GotoLabel(ActionTAG*atag, char* label);
 ActionTAG* action_WaitForFrame2(ActionTAG*atag, U8 skip);
 ActionTAG* action_With(ActionTAG*atag, char*object);
-ActionTAG* action_PushString(ActionTAG*atag, char*str);
+ActionTAG* action_PushString(ActionTAG*atag, const char*str);
 ActionTAG* action_PushFloat(ActionTAG*atag, float f);
 ActionTAG* action_PushNULL(ActionTAG*atag);
 ActionTAG* action_PushRegister(ActionTAG*atag, U8 reg);
@@ -909,11 +892,25 @@ void action_fixjump(ActionMarker m1, ActionMarker m2);
 
 extern char*blendModeNames[];
 
-int swf_ObjectPlace(TAG * t,U16 id,U16 depth,MATRIX * m,CXFORM * cx,U8 * name);
-int swf_ObjectPlaceClip(TAG * t,U16 id,U16 depth,MATRIX * m,CXFORM * cx,U8 * name, U16 clipaction);
-int swf_ObjectPlaceBlend(TAG * t,U16 id,U16 depth,MATRIX * m,CXFORM * cx,U8 * name, U8 blendmode);
+int swf_ObjectPlace(TAG * t,U16 id,U16 depth,MATRIX * m,CXFORM * cx,const U8 * name);
+int swf_ObjectPlaceClip(TAG * t,U16 id,U16 depth,MATRIX * m,CXFORM * cx,const U8 * name, U16 clipaction);
+int swf_ObjectPlaceBlend(TAG * t,U16 id,U16 depth,MATRIX * m,CXFORM * cx,const U8 * name, U8 blendmode);
 int swf_ObjectMove(TAG * t,U16 depth,MATRIX * m,CXFORM * cx);
 
+#define PF_MOVE         0x01
+#define PF_CHAR         0x02
+#define PF_MATRIX       0x04
+#define PF_CXFORM       0x08
+#define PF_RATIO        0x10
+#define PF_NAME         0x20
+#define PF_CLIPDEPTH    0x40
+#define PF_ACTIONEVENT  0x80
+
+#define PF2_FILTERS      0x01
+#define PF2_BLENDMODE    0x02
+#define PF2_ASBITMAP     0x04
+//...
+
 typedef struct _SWFPLACEOBJECT {
     U16 depth;
     U16 id; // may be 0
@@ -965,9 +962,84 @@ void swf_SetVideoStreamPFrame(TAG*tag, VIDEOSTREAM*s, RGBA*pic, int quant/* 1-31
 void swf_SetVideoStreamMover(TAG*tag, VIDEOSTREAM*s, signed char* movex, signed char* movey, void** image, int quant);
 void swf_VideoStreamClear(VIDEOSTREAM*stream);
 
+// swfbutton.c
+
+// Button States
+
+#define BS_HIT          0x08
+#define BS_DOWN         0x04
+#define BS_OVER         0x02
+#define BS_UP           0x01
+
+// Button Conditions
+
+/* missing: IDLE_OUTDOWN
+            OUTDOWN_OVERUP
+	    OVERUP_OUTDOWN
+*/
+#define BC_OVERDOWN_IDLE        0x0100
+#define BC_IDLE_OVERDOWN        0x0080
+#define BC_OUTDOWN_IDLE         0x0040
+#define BC_OUTDOWN_OVERDOWN     0x0020
+#define BC_OVERDOWN_OUTDOWN     0x0010
+#define BC_OVERDOWN_OVERUP      0x0008
+#define BC_OVERUP_OVERDOWN      0x0004
+#define BC_OVERUP_IDLE          0x0002
+#define BC_IDLE_OVERUP          0x0001
+
+#define BC_KEY(c) (c<<9)
+
+#define BC_CURSORLEFT           0x0200
+#define BC_CURSORRIGHT          0x0400
+#define BC_POS1                 0x0600
+#define BC_END                  0x0800
+#define BC_INSERT               0x0a00
+#define BC_DELETE               0x0c00
+#define BC_CLEAR		0x0e00
+#define BC_BACKSPACE            0x1000
+#define BC_ENTER                0x1a00
+#define BC_CURSORUP             0x1c00
+#define BC_CURSORDOWN           0x1e00
+#define BC_PAGEUP               0x2000
+#define BC_PAGEDOWN             0x2200
+#define BC_TAB                  0x2400
+#define BC_ESCAPE		0x3600
+#define BC_SPACE                0x4000
+
+/* these are probably only valid with linux:
+   Ctrl-A	 0x0200
+   Ctrl-X	 0x3000
+   Ctrl-Y	 0x3200
+   Ctrl-Z	 0x3400
+   Escape/Ctrl-[ 0x3600
+   Ctrl-\	 0x3800
+   Ctrl-]	 0x3a00
+   Ctrl-^	 0x3c00
+   Ctrl-/	 0x3e00
+   */
+
+/* everything above 0x4000 is standard ascii:
+   0x4000 ' ' 0x4200 '!' 0x4600 '#' 0x4800 '$' 0x4a00 '%' 0x4c00 '&' ...
+   0x6000 '0' ... 0x7200 '9'
+   0x8000 '@'
+   0x8200 'A' ...  0xb400 'Z'
+   ...
+   0xfc00 '~'
+ */
+
+// Button Flag
+
+#define BF_TRACKMENU            0x01
+
+int swf_ButtonSetRecord(TAG * t,U8 state,U16 id,U16 layer,MATRIX * m,CXFORM * cx);
+int swf_ButtonSetCondition(TAG * t,U16 condition); // for DefineButton2
+int swf_ButtonSetFlags(TAG * t,U8 flags);  // necessary for DefineButton2
+int swf_ButtonPostProcess(TAG * t,int anz_action); // Set all offsets in DefineButton2-Tags (how many conditions to process)
+ActionTAG* swf_ButtonGetAction(TAG*t);
+
 // swfrender.c
 
-typedef struct RENDERBUF 
+typedef struct RENDERBUF
 {
     int width;
     int height;
@@ -981,7 +1053,7 @@ void swf_Render_SetBackgroundColor(RENDERBUF*buf, RGBA color);
 RGBA* swf_Render(RENDERBUF*dest);
 void swf_RenderShape(RENDERBUF*dest, SHAPE2*shape, MATRIX*m, CXFORM*c, U16 depth,U16 clipdepth);
 void swf_RenderSWF(RENDERBUF*buf, SWF*swf);
-void swf_Render_AddImage(RENDERBUF*buf, U16 id, RGBA*img, int width, int height);
+void swf_Render_AddImage(RENDERBUF*buf, U16 id, RGBA*img, int width, int height); /* img is non-premultiplied */
 void swf_Render_ClearCanvas(RENDERBUF*dest);
 void swf_Render_Delete(RENDERBUF*dest);
 
@@ -995,7 +1067,7 @@ void swf_Render_Delete(RENDERBUF*dest);
 #define FILTERTYPE_CONVOLUTION 5
 #define FILTERTYPE_COLORMATRIX 6
 #define FILTERTYPE_GRADIENTBEVEL 7
-	
+
 extern char*filtername[];
 
 typedef struct _FILTER_GRADIENTGLOW {
@@ -1050,13 +1122,25 @@ typedef struct _FILTER_BLUR {
     int passes;
 } FILTER_BLUR;
 
+typedef struct _FILTER_GLOW {
+    U8 type;
+    RGBA rgba;
+    double blurx;
+    double blury;
+    double strength;
+    int passes;
+    char innerglow;
+    char knockout;
+    char composite;
+} FILTER_GLOW;
+
 void swf_SetFilter(TAG*tag, FILTER*f);
 FILTER*swf_GetFilter(TAG*tag);
 FILTER*swf_NewFilter(U8 type);
+void swf_DeleteFilter(FILTER*f);
 
 #ifdef __cplusplus
 }
 #endif
 
 #endif
-
