@@ -40,6 +40,8 @@ struct config_t
    char cat;
    char merge;
    char isframe;
+   char local_with_networking;
+   char local_with_filesystem;
    int loglevel;
    int sizex;
    char hassizex;
@@ -171,6 +173,16 @@ int args_callback_option(char*name,char*val) {
 	config.scaley = atoi(val)/100.0;
 	return 1;
     }
+    else if (!strcmp(name, "N"))
+    {
+	config.local_with_networking = 1;
+	return 0;
+    }
+    else if (!strcmp(name, "L"))
+    {
+	config.local_with_filesystem = 1;
+	return 0;
+    }
     else if (!strcmp(name, "t") || !strcmp(name, "T"))
     {
 	if(master_filename) {
@@ -212,6 +224,8 @@ static struct options_t options[] = {
 {"r", "rate"},
 {"X", "width"},
 {"Y", "height"},
+{"N", "local-with-networking"},
+{"L", "local-with-filesystem"},
 {"z", "zlib"},
 {0,0}
 };
@@ -289,10 +303,26 @@ void args_callback_usage(char *name)
     printf("-r , --rate <fps>              Set movie framerate to <fps> (frames/sec)\n");
     printf("-X , --width <width>           Force movie bbox width to <width> (default: use master width (not with -t))\n");
     printf("-Y , --height <height>          Force movie bbox height to <height> (default: use master height (not with -t))\n");
+    printf("-N , --local-with-networking     Make output file \"local-with-networking\"\n");
+    printf("-L , --local-with-filesystem     Make output file \"local-with-filesystem\"\n");
     printf("-z , --zlib <zlib>             Enable Flash 6 (MX) Zlib Compression\n");
     printf("\n");
 }
 
+void removeCommonTags(SWF * swf)
+{
+    TAG*tag = swf->firstTag;
+    while(tag) {
+	if(tag->id == ST_SCENEDESCRIPTION ||
+	   tag->id == ST_FILEATTRIBUTES ||
+	   tag->id == ST_REFLEX) {
+	    tag = swf_DeleteTag(swf, tag);
+	} else {
+	    tag = tag->next;
+	}
+    }
+}
+
 static void makestackmaster(SWF*swf)
 {
     TAG*tag;
@@ -300,6 +330,7 @@ static void makestackmaster(SWF*swf)
     SRECT box;
     int fileversion = config.zlib?6:3;
     int frameRate = 256;
+    U32 fileAttributes = 0;
     RGBA rgb;
     rgb.r=rgb.b=rgb.g=0;
     memset(&box, 0, sizeof(box));
@@ -317,6 +348,9 @@ static void makestackmaster(SWF*swf)
 	}
 	close(fi);
 	swf_RemoveJPEGTables(&head);
+        fileAttributes |= head.fileAttributes;
+	removeCommonTags(&head);
+
 	msg("<verbose> File %s has bounding box %d:%d:%d:%d\n",
 		slave_filename[t], 
 		head.movieSize.xmin, head.movieSize.ymin,
@@ -356,6 +390,7 @@ static void makestackmaster(SWF*swf)
     swf->fileVersion = fileversion;
     swf->movieSize = box;
     swf->frameRate = frameRate;
+    swf->fileAttributes = fileAttributes;
 
     swf->firstTag = swf_InsertTag(0, ST_SETBACKGROUNDCOLOR);
     tag = swf->firstTag;
@@ -443,7 +478,7 @@ void jpeg_assert(SWF*master, SWF*slave)
 	{
 	    // ok, both have jpegtables, but they're identical.
 	    // delete one and don't throw an error
-	    swf_DeleteTag(spos);
+	    swf_DeleteTag(slave, spos);
 	    spos = 0;
 	}
     }
@@ -931,9 +966,7 @@ void catcombine(SWF*master, char*slave_name, SWF*slave, SWF*newswf)
     }
     tag = swf_InsertTag(tag, ST_END);
 
-    tag = newswf->firstTag;
-    newswf->firstTag = newswf->firstTag->next; //remove temporary tag
-    swf_DeleteTag(tag);
+    swf_DeleteTag(newswf, tag);
 }
 
 void normalcombine(SWF*master, char*slave_name, SWF*slave, SWF*newswf)
@@ -1005,8 +1038,12 @@ void normalcombine(SWF*master, char*slave_name, SWF*slave, SWF*newswf)
 
     if (spriteid<0 && !config.isframe) {
 	if(slavename) {
-	    if(strcmp(slavename,"!!dummy!!"))
+	    if(strcmp(slavename,"!!dummy!!")) {
 		msg("<warning> Didn't find anything named %s in file. No substitutions will occur.", slavename);
+		if(!strcmp(slavename, "swf")) {
+		    msg("<warning> (If you were trying to combine rfxview with a document, try replacing 'swf' with 'viewport'.");
+		}
+	    }
 	}
 	else
 	    msg("<warning> Didn't find id %d in file. No substitutions will occur.", slaveid);
@@ -1045,9 +1082,7 @@ void normalcombine(SWF*master, char*slave_name, SWF*slave, SWF*newswf)
 		FLAGS_WRITEDEFINES|FLAGS_WRITENONDEFINES|   FLAGS_WRITESPRITE	);
     }
 
-    tag = newswf->firstTag;
-    newswf->firstTag = newswf->firstTag->next; //remove temporary tag
-    swf_DeleteTag(tag);
+    swf_DeleteTag(newswf, newswf->firstTag);
 }
 
 void combine(SWF*master, char*slave_name, SWF*slave, SWF*newswf)
@@ -1058,6 +1093,8 @@ void combine(SWF*master, char*slave_name, SWF*slave, SWF*newswf)
 
     if(!master->fileVersion && slave)
 	master->fileVersion = slave->fileVersion;
+        
+    master->fileAttributes |= slave->fileAttributes;
 
     swf_FoldAll(master);
     swf_FoldAll(slave);
@@ -1186,6 +1223,7 @@ int main(int argn, char *argv[])
 	    exit(1);
 	}
 	swf_RemoveJPEGTables(&master);
+	removeCommonTags(&master);
 	msg("<debug> Read %d bytes from masterfile\n", ret);
 	close(fi);
     }
@@ -1254,6 +1292,7 @@ int main(int argn, char *argv[])
 		msg("<debug> Read %d bytes from slavefile\n", ret);
 		close(fi);
 		swf_RemoveJPEGTables(&slave);
+		removeCommonTags(&slave);
 	    }
 	    else
 	    {
@@ -1278,14 +1317,20 @@ int main(int argn, char *argv[])
     if(!newswf.fileVersion)
 	newswf.fileVersion = 4;
 
+    if(config.local_with_filesystem)
+        newswf.fileAttributes &= ~FILEATTRIBUTE_USENETWORK;
+    if(config.local_with_networking)
+        newswf.fileAttributes |= FILEATTRIBUTE_USENETWORK;
+
     fi = open(outputname, O_BINARY|O_RDWR|O_TRUNC|O_CREAT, 0777);
 
     if(config.zlib) {
 	if(newswf.fileVersion < 6)
 	    newswf.fileVersion = 6;
-	swf_WriteSWC(fi, &newswf);
+        newswf.compressed = 1;
+	swf_WriteSWF(fi, &newswf);
     } else {
-	newswf.compressed = 0;
+	newswf.compressed = -1; // don't compress
 	swf_WriteSWF(fi, &newswf);
     }
     close(fi);
