@@ -59,6 +59,8 @@ static int hex = 0;
 static int used = 0;
 static int bbox = 0;
 static int cumulative = 0;
+static int showfonts = 0;
+static int showbuttons = 0;
 
 static struct options_t options[] = {
 {"h", "help"},
@@ -69,7 +71,9 @@ static struct options_t options[] = {
 {"a", "action"},
 {"t", "text"},
 {"s", "shapes"},
+{"F", "fonts"},
 {"p", "placements"},
+{"B", "buttons"},
 {"b", "bbox"},
 {"X", "width"},
 {"Y", "height"},
@@ -131,6 +135,10 @@ int args_callback_option(char*name,char*val)
 	xy |= 8;
 	return 0;
     }
+    else if(name[0]=='F') {
+	showfonts = 1;
+	return 0;
+    }
     else if(name[0]=='d') {
 	hex = 1;
 	return 0;
@@ -143,8 +151,12 @@ int args_callback_option(char*name,char*val)
 	bbox = 1;
 	return 0;
     }
+    else if(name[0]=='B') {
+	showbuttons = 1;
+	return 0;
+    }
     else if(name[0]=='D') {
-	action = placements = showtext = showshapes = 1;
+	showbuttons = action = placements = showtext = showshapes = 1;
 	return 0;
     }
     else {
@@ -171,7 +183,9 @@ void args_callback_usage(char *name)
     printf("-a , --action                  Disassemble action tags\n");
     printf("-t , --text                    Show text fields (like swfstrings).\n");
     printf("-s , --shapes                  Show shape coordinates/styles\n");
+    printf("-F , --fonts                   Show font information\n");
     printf("-p , --placements              Show placement information\n");
+    printf("-B , --buttons                 Show button information\n");
     printf("-b , --bbox                    Print tag's bounding boxes\n");
     printf("-X , --width                   Prints out a string of the form \"-X width\".\n");
     printf("-Y , --height                  Prints out a string of the form \"-Y height\".\n");
@@ -243,6 +257,7 @@ void dumpButton2Actions(TAG*tag, char*prefix)
 void dumpButtonActions(TAG*tag, char*prefix)
 {
     ActionTAG*actions;
+    swf_SetTagPos(tag, 0);
     swf_GetU16(tag); // id
     while (swf_GetU8(tag))      // state  -> parse ButtonRecord
     { swf_GetU16(tag);          // id
@@ -251,6 +266,113 @@ void dumpButtonActions(TAG*tag, char*prefix)
     }
     actions = swf_ActionGet(tag);
     swf_DumpActions(actions, prefix);
+    swf_ActionFree(actions);
+}
+
+void dumpButton(TAG*tag, char*prefix)
+{
+    swf_SetTagPos(tag, 0);
+    swf_GetU16(tag); // id
+    while (1) {
+        U8 flags = swf_GetU8(tag);
+        if(!flags)
+            break;
+        U16 id = swf_GetU16(tag);
+        U16 depth = swf_GetU16(tag);
+        char event[80];
+        sprintf(event, "%s%s%s%s", 
+                (flags&BS_HIT)?"[hit]":"",
+                (flags&BS_DOWN)?"[down]":"",
+                (flags&BS_OVER)?"[over]":"",
+                (flags&BS_UP)?"[up]":"");
+        if(flags&0xf0) {
+            printf("%s | Show %d at depth %d for %s flags=%02x\n", prefix, id, depth, event, flags);
+        } else {
+            printf("%s | Show %d at depth %d for %s\n", prefix, id, depth, event);
+        }
+        swf_GetMatrix(tag,NULL);  // matrix
+    }
+}
+
+void dumpFont(TAG*tag, char*prefix)
+{
+    SWFFONT* font = malloc(sizeof(SWFFONT));
+    memset(font, 0, sizeof(SWFFONT));
+    if(tag->id == ST_DEFINEFONT2) {
+	swf_FontExtract_DefineFont2(0, font, tag);
+    } else if(tag->id == ST_DEFINEFONT) {
+	swf_FontExtract_DefineFont(0, font, tag);
+    } else {
+	printf("%sCan't parse %s yet\n", prefix,swf_TagGetName(tag));
+    }
+    printf("%sID: %d\n", prefix,font->id);
+    printf("%sVersion: %d\n", prefix,font->version);
+    printf("%sname: %s\n", prefix,font->name);
+    printf("%scharacters: %d\n", prefix,font->numchars);
+    printf("%shightest mapped unicode value: %d\n", prefix,font->maxascii);
+    if(font->layout)
+    {
+	printf("%sascent:%.2f\n", prefix,font->layout->ascent / 20.0);
+	printf("%sdescent:%.2f\n", prefix,font->layout->descent / 20.0);
+	printf("%sleading:%.2f\n", prefix,font->layout->leading / 20.0);
+	printf("%skerning records:%d\n", prefix,font->layout->kerningcount);
+    }
+    printf("%sstyle: %d\n", prefix,font->style);
+    printf("%sencoding: %02x\n", prefix,font->encoding);
+    printf("%slanguage: %02x\n", prefix,font->language);
+    int t;
+    for(t=0;t<font->numchars;t++) {
+	int u = font->glyph2ascii?font->glyph2ascii[t]:-1;
+	printf("%s== Glyph %d: advance=%d encoding=%d ==\n", prefix, t, font->glyph[t].advance, u);
+	SHAPE2* shape = swf_ShapeToShape2(font->glyph[t].shape);
+	SHAPELINE*line = shape->lines;
+
+	while(line) {
+	    if(line->type == moveTo) {
+		printf("%smoveTo %.2f %.2f\n", prefix, line->x/20.0, line->y/20.0);
+	    } else if(line->type == lineTo) {
+		printf("%slineTo %.2f %.2f\n", prefix, line->x/20.0, line->y/20.0);
+	    } else if(line->type == splineTo) {
+		printf("%ssplineTo (%.2f %.2f) %.2f %.2f\n", prefix,
+			line->sx/20.0, line->sy/20.0,
+			line->x/20.0, line->y/20.0
+			);
+	    }
+	    line = line->next;
+	}
+	swf_Shape2Free(shape);
+	free(shape);
+    }
+    
+    /*
+      not part of the file
+
+    printf("%sencoding table:", prefix, prefix);
+    char filled0=0, lastfilled=0;
+    for(t=0;t<font->maxascii;t++) {
+        if((t&15)==0) {
+            printf("\n%s%08x ", prefix, t);
+            int s;
+            if(!filled0 && t) {
+                for(s=t;s<font->maxascii;s++) {
+                    if(font->ascii2glyph[s]>=0) break;
+                }
+                if(s>t+32) {
+                    printf("*");
+                    t = ((s-16)&~15)-1;
+                    continue;
+                }
+            }
+            filled0 = 0;
+            for(s=t;s<t+16 && s<font->maxascii;s++) {
+                if(font->ascii2glyph[s]>=0) filled0=1;
+            }
+        }
+        printf("%4d ", font->ascii2glyph[t]);
+    }
+    printf("\n");*/
+
+    swf_FontFree(font);
 }
 
 SWF swf;
@@ -260,7 +382,7 @@ SWFFONT**fonts;
 void textcallback(void*self, int*glyphs, int*ypos, int nr, int fontid, int fontsize, int startx, int starty, RGBA*color) 
 {
     int font=-1,t;
-    printf("                <%2d glyphs in font %2d, color #%02x%02x%02x%02x> ",nr, fontid, color->r, color->g, color->b, color->a);
+    printf("                <%2d glyphs in font %2d size %d, color #%02x%02x%02x%02x> ",nr, fontid, fontsize, color->r, color->g, color->b, color->a);
     for(t=0;t<fontnum;t++)
     {
 	if(fonts[t]->id == fontid) {
@@ -479,6 +601,15 @@ void dumpFilter(FILTER*filter)
 	FILTER_BLUR*f = (FILTER_BLUR*)filter;
 	printf("blurx: %f blury: %f\n", f->blurx, f->blury);
 	printf("passes: %d\n", f->passes);
+    } if(filter->type == FILTERTYPE_GLOW) {
+	FILTER_GLOW*f = (FILTER_GLOW*)filter;
+	printf("color %02x%02x%02x%02x\n", f->rgba.r,f->rgba.g,f->rgba.b,f->rgba.a);
+	printf("blurx: %f blury: %f strength: %f\n", f->blurx, f->blury, f->strength);
+	printf("passes: %d\n", f->passes);
+	printf("flags: %s%s%s\n", 
+		f->knockout?"knockout ":"",
+		f->composite?"composite ":"",
+		f->innerglow?"innerglow":"");
     } if(filter->type == FILTERTYPE_DROPSHADOW) {
 	FILTER_DROPSHADOW*f = (FILTER_DROPSHADOW*)filter;
 	printf("blurx: %f blury: %f\n", f->blurx, f->blury);
@@ -685,11 +816,13 @@ char* fillstyle2str(FILLSTYLE*style)
 	    /* TODO: display information about that bitmap */
 	    sprintf(stylebuf, "BITMAPt%s %d", (style->type&2)?"n":"", style->id_bitmap);
 	    /* TODO: show matrix */
+	    //swf_DumpMatrix(stdout, &style->m);
 	    break;
 	case 0x41: case 0x43:
 	    /* TODO: display information about that bitmap */
 	    sprintf(stylebuf, "BITMAPc%s %d", (style->type&2)?"n":"", style->id_bitmap);
 	    /* TODO: show matrix */
+	    //swf_DumpMatrix(stdout, &style->m);
 	    break;
 	default:
 	    sprintf(stylebuf, "UNKNOWN[%02x]",style->type);
@@ -732,6 +865,10 @@ void handleShape(TAG*tag, char*prefix)
 	    printf("%-2d) %s", t+1, linestyle2str(&shape.linestyles[t]));
 	}
 	printf("\n");
+        //if(shape.fillstyles[t].type&0x40) {
+        //    MATRIX m = shape.fillstyles[t].m;
+        //    swf_DumpMatrix(stdout, &m);
+        //}
     }
 
     printf("%s |\n", prefix);
@@ -817,7 +954,7 @@ void dumperror(const char* format, ...)
     va_list arglist;
 
     va_start(arglist, format);
-    vsprintf(buf, format, arglist);
+    vsnprintf(buf, sizeof(buf)-1, format, arglist);
     va_end(arglist);
 
     if(!html && !xy)
@@ -867,30 +1004,46 @@ int main (int argc,char ** argv)
     }
 
     f = open(filename,O_RDONLY|O_BINARY);
-
     if (f<0)
     { 
 	char buffer[256];
-	sprintf(buffer, "Couldn't open %s", filename);
+	sprintf(buffer, "Couldn't open %.200s", filename);
         perror(buffer);
         exit(1);
     }
-    if FAILED(swf_ReadSWF(f,&swf))
-    { 
-        fprintf(stderr, "%s is not a valid SWF file or contains errors.\n",filename);
-        close(f);
-        exit(1);
-    }
+    char header[3];
+    read(f, header, 3);
+    char compressed = (header[0]=='C');
+    char isflash = (header[0]=='F' && header[1] == 'W' && header[2] == 'S') ||
+                   (header[0]=='C' && header[1] == 'W' && header[2] == 'S');
+    close(f);
+
+    int fl=strlen(filename);
+    if(!isflash && fl>3 && !strcmp(&filename[fl-4], ".abc")) {
+        swf_ReadABCfile(filename, &swf);
+    } else {
+        f = open(filename,O_RDONLY|O_BINARY);
+        if FAILED(swf_ReadSWF(f,&swf))
+        { 
+            fprintf(stderr, "%s is not a valid SWF file or contains errors.\n",filename);
+            close(f);
+            exit(1);
+        }
 
 #ifdef HAVE_STAT
-    fstat(f, &statbuf);
-    if(statbuf.st_size != swf.fileSize && !swf.compressed)
-        dumperror("Real Filesize (%d) doesn't match header Filesize (%d)",
-                statbuf.st_size, swf.fileSize);
-    filesize = statbuf.st_size;
+        fstat(f, &statbuf);
+        if(statbuf.st_size != swf.fileSize && !compressed)
+            dumperror("Real Filesize (%d) doesn't match header Filesize (%d)",
+                    statbuf.st_size, swf.fileSize);
+        filesize = statbuf.st_size;
 #endif
+        close(f);
+    }
 
-    close(f);
+    //if(action && swf.fileVersion>=9) {
+    //    fprintf(stderr, "Actionscript parsing (-a) not yet supported for SWF versions>=9\n");
+    //    action = 0;
+    //}
 
     xsize = (swf.movieSize.xmax-swf.movieSize.xmin)/20;
     ysize = (swf.movieSize.ymax-swf.movieSize.ymin)/20;
@@ -923,9 +1076,9 @@ int main (int argc,char ** argv)
     if(html)
     {
 	char*fileversions[] = {"","1,0,0,0", "2,0,0,0","3,0,0,0","4,0,0,0",
-			       "5,0,0,0","6,0,23,0","7,0,0,0","8,0,0,0","9,0,0,0"};
-	if(swf.fileVersion>9) {
-	    fprintf(stderr, "Fileversion>9\n");
+			       "5,0,0,0","6,0,23,0","7,0,0,0","8,0,0,0","9,0,0,0","10,0,0,0", "11,0,0,0", "12,0,0,0"};
+	if(swf.fileVersion>10) {
+	    fprintf(stderr, "Fileversion>10\n");
 	    exit(1);
 	}
 
@@ -948,9 +1101,11 @@ int main (int argc,char ** argv)
 		   "  <PARAM NAME=\"PLAY\" VALUE=\"true\">\n" 
 		   "  <PARAM NAME=\"LOOP\" VALUE=\"true\">\n"
 		   "  <PARAM NAME=\"QUALITY\" VALUE=\"high\">\n"
+		   "  <PARAM NAME=\"ALLOWSCRIPTACCESS\" VALUE=\"always\">\n"
 		   "  <EMBED SRC=\"%s\" WIDTH=\"%d\" HEIGHT=\"%d\"\n" //bgcolor=#ffffff?
 		   "   PLAY=\"true\" ALIGN=\"\" LOOP=\"true\" QUALITY=\"high\"\n"
 		   "   TYPE=\"application/x-shockwave-flash\"\n"
+                   "   ALLOWSCRIPTACCESS=\"always\"\n"
 		   "   PLUGINSPAGE=\"http://www.macromedia.com/go/getflashplayer\">\n"
 		   "  </EMBED>\n" 
 		   "</OBJECT>\n", xsize, ysize, fileversions[swf.fileVersion], 
@@ -959,7 +1114,7 @@ int main (int argc,char ** argv)
 	return 0;
     } 
     printf("[HEADER]        File version: %d\n", swf.fileVersion);
-    if(swf.compressed) {
+    if(compressed) {
 	printf("[HEADER]        File is zlib compressed.");
 	if(filesize && swf.fileSize)
 	    printf(" Ratio: %02d%%\n", filesize*100/(swf.fileSize));
@@ -1062,6 +1217,32 @@ int main (int argc,char ** argv)
         else if(tag->id == ST_FREECHARACTER) {
             printf(" frees object %04d", swf_GetPlaceID(tag));
         }
+        else if(tag->id == ST_FILEATTRIBUTES) {
+            swf_SetTagPos(tag, 0);
+            U32 flags = swf_GetU32(tag);
+            if(flags&1) printf(" usenetwork");
+            if(flags&8) printf(" as3");
+            if(flags&16) printf(" symbolclass");
+            if(flags&~(1|8|16))
+                printf(" flags=%02x", flags);
+        }
+        else if(tag->id == ST_DOABC) {
+            swf_SetTagPos(tag, 0);
+            U32 flags = swf_GetU32(tag);
+            char*s = swf_GetString(tag);
+            if(flags&~1) {
+                printf(" flags=%08x", flags);
+            }
+            if(*s) {
+                printf(" \"%s\"", s);
+            }
+            if(flags&1) {
+                if(name)
+                    printf(",");
+                printf(" lazy load");
+            }
+            swf_SetTagPos(tag, 0);
+        }
 	else if(tag->id == ST_STARTSOUND) {
 	    U8 flags;
 	    U16 id;
@@ -1121,18 +1302,15 @@ int main (int argc,char ** argv)
 	    if(issprite) {spriteframe++; spriteframelabel = 0;}
 	    if(!issprite) {mainframe++; framelabel = 0;}
 	}
-
-	if(tag->id == ST_SETBACKGROUNDCOLOR) {
+        else if(tag->id == ST_SETBACKGROUNDCOLOR) {
 	    U8 r = swf_GetU8(tag);
 	    U8 g = swf_GetU8(tag);
 	    U8 b = swf_GetU8(tag);
-	    printf(" (%02x/%02x/%02x)\n",r,g,b);
+	    printf(" (%02x/%02x/%02x)",r,g,b);
 	}
 	else if(tag->id == ST_PROTECT) {
 	    if(tag->len>0) {
-		printf(" %s\n", swf_GetString(tag));
-	    } else {
-		printf("\n");
+		printf(" %s", swf_GetString(tag));
 	    }
 	}
 	else if(tag->id == ST_CSMTEXTSETTINGS) {
@@ -1152,10 +1330,11 @@ int main (int argc,char ** argv)
 		printf("unknown[%08x],", flags);
 	    float thickness = swf_GetFixed(tag);
 	    float sharpness = swf_GetFixed(tag);
-	    printf("s=%.2f,t=%.2f)\n", thickness, sharpness);
+	    printf("s=%.2f,t=%.2f)", thickness, sharpness);
 	    swf_GetU8(tag);
 	}
-	else if(tag->id == ST_DEFINEBITSLOSSLESS ||
+
+	if(tag->id == ST_DEFINEBITSLOSSLESS ||
 	   tag->id == ST_DEFINEBITSLOSSLESS2) {
 	    handleDefineBits(tag);
 	    printf("\n");
@@ -1187,9 +1366,15 @@ int main (int argc,char ** argv)
 	    else
 		printf("\n");
 	}
+	else if(tag->id == ST_DEFINESCALINGGRID) {
+	    U16 id = swf_GetU16(tag);
+	    SRECT r;
+	    swf_GetRect(tag, &r);
+	    printf(" (%.2f,%.2f)-(%.2f,%.2f)\n", r.xmin/20.0, r.ymin/20.0, r.xmax/20.0, r.ymax/20.0);
+	}
 	else if(tag->id == ST_PLACEOBJECT2 || tag->id == ST_PLACEOBJECT3) {
 	}
-	else if(tag->id == ST_NAMECHARACTER) {
+	else if(tag->id == ST_NAMECHARACTER || tag->id==ST_DEFINEFONTNAME) {
 	    swf_GetU16(tag);
 	    printf(" \"%s\"\n", swf_GetString(tag));
 	}
@@ -1224,7 +1409,7 @@ int main (int argc,char ** argv)
 	    if(tag->len)
 		dumperror("End Tag not empty");
         }
-	else if(tag->id == ST_EXPORTASSETS) {
+	else if(tag->id == ST_EXPORTASSETS || tag->id == ST_SYMBOLCLASS) {
 	    handleExportAssets(tag, myprefix);
 	}
         else if(tag->id == ST_DOACTION && action) {
@@ -1232,17 +1417,32 @@ int main (int argc,char ** argv)
             actions = swf_ActionGet(tag);
             swf_DumpActions(actions, myprefix);
         }
+        else if((tag->id == ST_DOABC || tag->id == ST_RAWABC) && action) {
+            void*abccode = swf_ReadABC(tag);
+            swf_DumpABC(stdout, abccode, "");
+            swf_FreeABC(abccode);
+        }
         else if(tag->id == ST_DOINITACTION && action) {
             ActionTAG*actions;
             swf_GetU16(tag); // id
             actions = swf_ActionGet(tag);
             swf_DumpActions(actions, myprefix);
         }
-	else if(tag->id == ST_DEFINEBUTTON && action) {
-	    dumpButtonActions(tag, myprefix);
+	else if(tag->id == ST_DEFINEBUTTON) {
+            if(showbuttons) {
+                dumpButton(tag, myprefix);
+            }
+            if(action) {
+	        dumpButtonActions(tag, myprefix);
+            }
 	}
-	else if(tag->id == ST_DEFINEBUTTON2 && action) {
-	    dumpButton2Actions(tag, myprefix);
+	else if(swf_isFontTag(tag) && showfonts) {
+	    dumpFont(tag, myprefix);
+	}
+	else if(tag->id == ST_DEFINEBUTTON2) {
+            if(action) {
+	        dumpButton2Actions(tag, myprefix);
+            }
 	}
 	else if(tag->id == ST_PLACEOBJECT) {
 	    handlePlaceObject(tag, myprefix);
@@ -1250,9 +1450,17 @@ int main (int argc,char ** argv)
 	else if(tag->id == ST_PLACEOBJECT2 || tag->id == ST_PLACEOBJECT3) {
 	    handlePlaceObject23(tag, myprefix);
 	}
+	else if(tag->id == ST_DEFINEFONTNAME) {
+	    swf_SetTagPos(tag, 0);
+	    swf_GetU16(tag); //id
+	    swf_GetString(tag); //name
+	    char* copyright = swf_GetString(tag);
+	    printf("%s%s\n", myprefix, copyright);
+	}
 	else if(tag->id == ST_DEFINESHAPE ||
 		tag->id == ST_DEFINESHAPE2 ||
-		tag->id == ST_DEFINESHAPE3) {
+		tag->id == ST_DEFINESHAPE3 ||
+		tag->id == ST_DEFINESHAPE4) {
 	    if(showshapes)
 		handleShape(tag, myprefix);
 	}
